@@ -30,17 +30,17 @@
 ;;;   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 ;;;   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ;;;  
-;;;  $Id: auxsys.scm,v 1.8 2003/10/06 12:18:50 shirok Exp $
+;;;  $Id: auxsys.scm,v 1.10 2004/04/11 05:29:27 fuyuki Exp $
 ;;;
 
 (define-module gauche.auxsys
   (export fmod frexp modf ldexp
-          sys-abort sys-mkfifo
+          sys-abort sys-realpath sys-mkfifo
           sys-setgid sys-setpgid sys-getpgid sys-getpgrp
           sys-setsid sys-setuid sys-times sys-uname sys-ctermid
           sys-gethostname sys-getdomainname
           sys-putenv sys-setenv sys-unsetenv
-          sys-gettimeofday sys-chown sys-utime
+          sys-gettimeofday sys-chown sys-lchown sys-utime
           sys-getgroups sys-getlogin sys-localeconv)
   )
 (select-module gauche.auxsys)
@@ -49,6 +49,11 @@
 
 ;; define alternatives if the platform doesn't support...
 
+(define sys-realpath
+  (if (symbol-bound? '%sys-realpath)
+      %sys-realpath
+      (lambda (path) (error "sys-realpath not supported on this platform"))))
+
 (define sys-gethostname
   (if (symbol-bound? '%sys-gethostname)
     %sys-gethostname
@@ -67,7 +72,8 @@
 (define sys-setenv
   (if (symbol-bound? '%sys-setenv)
     %sys-setenv
-    (lambda (var val val) (error "sys-setenv not supported on this platform"))))
+    (lambda (var val overwrite)
+      (error "sys-setenv not supported on this platform"))))
 
 (define sys-unsetenv
   (if (symbol-bound? '%sys-unsetenv)
@@ -92,4 +98,10 @@
     %sys-gettimeofday
     (lambda () (values (sys-time) 0))))
 
+(define sys-lchown
+  (if (symbol-bound? '%sys-lchown)
+      %sys-lchown
+      (lambda (path owner group)
+        (error "sys-lchown not supported on this platform"))))
+
 (provide "gauche/auxsys")
