@@ -30,7 +30,7 @@
  *   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  *   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- *  $Id: charconv.c,v 1.46 2003/12/08 08:32:58 shirok Exp $
+ *  $Id: charconv.c,v 1.50 2004/11/06 12:43:40 shirok Exp $
  */
 
 #include <string.h>
@@ -229,10 +229,10 @@ static int conv_input_filler(ScmPort *port, int mincnt)
     }
 }
 
-static int conv_input_closer(ScmPort *p)
+static void conv_input_closer(ScmPort *p)
 {
     ScmConvInfo *info = (ScmConvInfo*)p->src.buf.data;
-    return jconv_close(info);
+    jconv_close(info);
 }
 
 ScmObj Scm_MakeInputConversionPort(ScmPort *fromPort,
@@ -303,7 +303,20 @@ ScmObj Scm_MakeInputConversionPort(ScmPort *fromPort,
     bufrec.data = (void*)cinfo;
 
     name = conv_name(SCM_PORT_INPUT, fromPort, fromCode, toCode);
-    return Scm_MakeBufferedPort(name, SCM_PORT_INPUT, TRUE, &bufrec);
+    return Scm_MakeBufferedPort(SCM_CLASS_PORT, name, SCM_PORT_INPUT, TRUE, &bufrec);
+}
+
+/* a special case of input conversion port --- coding-aware port coversion.
+   this function is called via Scm_CodingAwarePortHook from
+   src/port.c */
+
+static ScmPort *coding_aware_conv(ScmPort *src, const char *encoding)
+{
+    return SCM_PORT(Scm_MakeInputConversionPort(src,
+                                                encoding,
+                                                Scm_SupportedCharacterEncodings()[0],
+                                                SCM_FALSE,
+                                                0, TRUE));
 }
 
 /*------------------------------------------------------------
@@ -327,7 +340,7 @@ ScmObj Scm_MakeInputConversionPort(ScmPort *fromPort,
 
 #define GLIBC_2_1_ICONV_BUG
 
-static int conv_output_closer(ScmPort *port)
+static void conv_output_closer(ScmPort *port)
 {
     ScmConvInfo *info = (ScmConvInfo*)port->src.buf.data;
     int r;
@@ -356,7 +369,7 @@ static int conv_output_closer(ScmPort *port)
         Scm_ClosePort(info->remote);
         info->remoteClosed = TRUE;
     }
-    return jconv_close(info);
+    jconv_close(info);
 }
 
 static int conv_output_flusher(ScmPort *port, int cnt, int forcep)
@@ -465,7 +478,7 @@ ScmObj Scm_MakeOutputConversionPort(ScmPort *toPort,
     bufrec.data = (void*)cinfo;
     
     name = conv_name(SCM_PORT_OUTPUT, toPort, fromCode, toCode);
-    return Scm_MakeBufferedPort(name, SCM_PORT_OUTPUT, TRUE, &bufrec);
+    return Scm_MakeBufferedPort(SCM_CLASS_PORT, name, SCM_PORT_OUTPUT, TRUE, &bufrec);
 }
 
 /*------------------------------------------------------------
@@ -588,6 +601,8 @@ extern void Scm_Init_convlib(ScmModule *module);
 extern void Scm_Init_convguess(void);
 SCM_EXTERN ScmChar (*Scm_UcsToCharHook)(int ucs4);
 SCM_EXTERN int (*Scm_CharToUcsHook)(ScmChar ch);
+SCM_EXTERN ScmPort *(*Scm_CodingAwarePortHook)(ScmPort *src,
+                                               const char *encoding);
 
 void Scm_Init_libcharconv(void)
 {
@@ -612,4 +627,5 @@ void Scm_Init_libcharconv(void)
     Scm_Init_convlib(mod);
     Scm_UcsToCharHook = ucstochar;
     Scm_CharToUcsHook = chartoucs;
+    Scm_CodingAwarePortHook = coding_aware_conv;
 }
