@@ -1,7 +1,7 @@
 /*
  * addr.c - socket address
  *
- *  Copyright(C) 2001-2003 by Shiro Kawai (shiro@acm.org)
+ *  Copyright(C) 2001-2004 by Shiro Kawai (shiro@acm.org)
  *
  *  Permission to use, copy, modify, distribute this software and
  *  accompanying documentation for any purpose is hereby granted,
@@ -12,10 +12,10 @@
  *  warranty.  In no circumstances the author(s) shall be liable
  *  for any damages arising out of the use of this software.
  *
- *  $Id: addr.c,v 1.18 2004/01/28 00:28:22 fuyuki Exp $
+ *  $Id: addr.c,v 1.21 2004/10/09 11:36:36 shirok Exp $
  */
 
-#include "net.h"
+#include "gauche/net.h"
 #include <string.h>
 
 static ScmObj key_path = SCM_FALSE;
@@ -153,7 +153,7 @@ static ScmObj sockaddr_in_allocate(ScmClass *klass, ScmObj initargs)
     ScmObj host = Scm_GetKeyword(key_host, initargs, key_any);
     ScmObj port = Scm_GetKeyword(key_port, initargs, SCM_MAKE_INT(0));
     ScmSockAddrIn *addr;
-    
+
     if (!SCM_INTP(port)) {
         Scm_Error(":port parameter must be a small exact integer, but got %S",
                   port);
@@ -263,15 +263,11 @@ void Scm_Init_NetAddr(ScmModule *mod)
     key_broadcast = SCM_MAKE_KEYWORD("broadcast");
     key_loopback  = SCM_MAKE_KEYWORD("loopback");
 
-    Scm_InitBuiltinClass(&Scm_SockAddrClass, "<sockaddr>", NULL,
-                         sizeof(ScmSockAddr), mod);
-    Scm_InitBuiltinClass(&Scm_SockAddrUnClass, "<sockaddr-un>", NULL,
-                         sizeof(ScmSockAddrUn), mod);
-    Scm_InitBuiltinClass(&Scm_SockAddrInClass, "<sockaddr-in>", NULL,
-                         sizeof(ScmSockAddrIn), mod);
+    Scm_InitStaticClass(&Scm_SockAddrClass, "<sockaddr>", mod, NULL, 0);
+    Scm_InitStaticClass(&Scm_SockAddrUnClass, "<sockaddr-un>", mod, NULL, 0);
+    Scm_InitStaticClass(&Scm_SockAddrInClass, "<sockaddr-in>", mod, NULL, 0);
 #ifdef HAVE_IPV6
-    Scm_InitBuiltinClass(&Scm_SockAddrIn6Class, "<sockaddr-in6>", NULL,
-                         sizeof(ScmSockAddrIn6), mod);
+    Scm_InitStaticClass(&Scm_SockAddrIn6Class, "<sockaddr-in6>", mod, NULL, 0);
 #endif /* HAVE_IPV6 */
 }
 
