@@ -1,7 +1,7 @@
 /*
  * netdb.c - obtain information about network
  *
- *   Copyright (c) 2000-2003 Shiro Kawai, All rights reserved.
+ *   Copyright (c) 2000-2004 Shiro Kawai, All rights reserved.
  * 
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -30,10 +30,10 @@
  *   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  *   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- *  $Id: netdb.c,v 1.8 2004/01/28 00:28:22 fuyuki Exp $
+ *  $Id: netdb.c,v 1.11 2004/10/09 11:36:36 shirok Exp $
  */
 
-#include "net.h"
+#include "gauche/net.h"
 #include <gauche/class.h>
 
 #define DATA_BUFSIZ  980
@@ -146,7 +146,7 @@ ScmObj Scm_GetHostByAddr(const char *addr, int type)
         Scm_Error("bad inet address format: %s", addr);
     }
     
-#if defined(GETHOSTBYNAME_R_NUMARGS)
+#if defined(GETHOSTBYADDR_R_NUMARGS)
     {
         ScmObj entry = SCM_FALSE;
         int herr = 0, bufsiz = DATA_BUFSIZ;
@@ -176,7 +176,7 @@ ScmObj Scm_GetHostByAddr(const char *addr, int type)
         entry = SCM_OBJ(make_hostent(&he));
         return entry;
     }
-#else /* !defined(GETHOSTBYNAME_R_NUMARGS) */
+#else /* !defined(GETHOSTBYADDR_R_NUMARGS) */
     {
         volatile ScmObj entry = SCM_FALSE;
         struct hostent *he;
@@ -192,7 +192,7 @@ ScmObj Scm_GetHostByAddr(const char *addr, int type)
                          } while (0));
         return entry;
     }
-#endif /* !defined(GETHOSTBYNAME_R_NUMARGS) */
+#endif /* !defined(GETHOSTBYADDR_R_NUMARGS) */
 }
 
 static ScmObj hostent_name(ScmSysHostent *entry)
@@ -545,12 +545,12 @@ ScmObj Scm_GetNameinfo(ScmSockAddr *addr, int flags)
 
 void Scm_Init_NetDB(ScmModule *mod)
 {
-    Scm_InitBuiltinClass(&Scm_SysHostentClass, "<sys-hostent>",
-                         hostent_slots, sizeof(ScmSysHostent), mod);
-    Scm_InitBuiltinClass(&Scm_SysProtoentClass, "<sys-protoent>",
-                         protoent_slots, sizeof(ScmSysProtoent), mod);
-    Scm_InitBuiltinClass(&Scm_SysServentClass, "<sys-servent>",
-                         servent_slots, sizeof(ScmSysServent), mod);
+    Scm_InitStaticClass(&Scm_SysHostentClass, "<sys-hostent>", mod,
+                        hostent_slots, 0);
+    Scm_InitStaticClass(&Scm_SysProtoentClass, "<sys-protoent>", mod,
+                        protoent_slots, 0);
+    Scm_InitStaticClass(&Scm_SysServentClass, "<sys-servent>", mod,
+                        servent_slots, 0);
     SCM_INTERNAL_MUTEX_INIT(netdb_data.hostent_mutex);
     SCM_INTERNAL_MUTEX_INIT(netdb_data.protoent_mutex);
     SCM_INTERNAL_MUTEX_INIT(netdb_data.servent_mutex);
