@@ -1,7 +1,7 @@
 /*
  * termios.c - termios interface
  *
- *   Copyright (c) 2000-2003 Shiro Kawai, All rights reserved.
+ *   Copyright (c) 2000-2004 Shiro Kawai, All rights reserved.
  * 
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -30,7 +30,7 @@
  *   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  *   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- *  $Id: termios.c,v 1.11 2003/08/08 19:39:15 shirok Exp $
+ *  $Id: termios.c,v 1.15 2004/10/09 11:36:36 shirok Exp $
  */
 
 #include <string.h>
@@ -38,6 +38,8 @@
 #include <gauche/class.h>
 #include <gauche/extend.h>
 
+#if !defined(__MINGW32__)
+
 /*
  * Termios interface
  */
@@ -71,11 +73,29 @@ TERMIOS_GET_N_SET(c_oflag)
 TERMIOS_GET_N_SET(c_cflag)
 TERMIOS_GET_N_SET(c_lflag)
 
+static ScmObj termios_c_cc_get(ScmSysTermios* t)
+{
+    return Scm_MakeU8VectorFromArray(NCCS, (const unsigned char*)t->term.c_cc);
+}
+
+static void termios_c_cc_set(ScmSysTermios* t, ScmObj val)
+{
+    if (!SCM_U8VECTORP(val)) {
+        Scm_Error("cc type must be a u8vector, but got %S", val);
+    }
+    if (SCM_U8VECTOR_SIZE(val) != NCCS) {
+        Scm_Error("size of cc must be %u, but got %u",
+                  NCCS, SCM_U8VECTOR_SIZE(val));
+    }
+    memcpy(t->term.c_cc, SCM_U8VECTOR_ELEMENTS(val), NCCS);
+}
+
 static ScmClassStaticSlotSpec termios_slots[] = {
     SCM_CLASS_SLOT_SPEC("iflag", termios_c_iflag_get, termios_c_iflag_set),
     SCM_CLASS_SLOT_SPEC("oflag", termios_c_oflag_get, termios_c_oflag_set),
     SCM_CLASS_SLOT_SPEC("cflag", termios_c_cflag_get, termios_c_cflag_set),
     SCM_CLASS_SLOT_SPEC("lflag", termios_c_lflag_get, termios_c_lflag_set),
+    SCM_CLASS_SLOT_SPEC("cc", termios_c_cc_get, termios_c_cc_set),
     { NULL }
 };
 
@@ -120,6 +140,7 @@ ScmObj Scm_Forkpty(ScmObj slaveterm)
 }
 #endif /*HAVE_FORKPTY*/
 
+#endif /*!defined(__MINGW32) */
 
 /*
  * Initializaion
@@ -132,10 +153,12 @@ void Scm_Init_termios(void)
     ScmModule *mod;
     SCM_INIT_EXTENSION(termios);
     mod = SCM_MODULE(SCM_FIND_MODULE("gauche.termios", TRUE));
-    Scm_InitBuiltinClass(&Scm_SysTermiosClass, "<sys-termios>",
-                         termios_slots, sizeof(ScmSysTermios), mod);
     Scm_Init_termiolib(mod);
 
+#ifndef __MINGW32__
+    Scm_InitStaticClass(&Scm_SysTermiosClass, "<sys-termios>", mod,
+                        termios_slots, 0);
+
     /* Constants for termios.  Non-POSIX symbols are guarded by #ifdef's */
 #define DEFSYM(sym) \
     SCM_DEFINE(mod, #sym, Scm_MakeIntegerFromUI(sym))
@@ -285,6 +308,52 @@ void Scm_Init_termios(void)
     DEFSYM(PENDIN);
 #endif
 
+    /* c_cc size */
+    DEFSYM(NCCS);
+    
+    /* disable character */
+    DEFSYM(_POSIX_VDISABLE);
+
+    /* c_cc subscripts */
+    DEFSYM(VEOF);
+    DEFSYM(VEOL);
+    DEFSYM(VERASE);
+    DEFSYM(VINTR);
+    DEFSYM(VKILL);
+    DEFSYM(VMIN);
+    DEFSYM(VQUIT);
+    DEFSYM(VSTART);
+    DEFSYM(VSTOP);
+    DEFSYM(VSUSP);
+    DEFSYM(VTIME);
+#ifdef VDISCARD
+    DEFSYM(VDISCARD);
+#endif
+#ifdef VDSUSP
+    DEFSYM(VDSUSP);
+#endif
+#ifdef EOL2
+    DEFSYM(VEOL2);
+#endif
+#ifdef LNEXT
+    DEFSYM(VLNEXT);
+#endif
+#ifdef VREPRINT
+    DEFSYM(VREPRINT);
+#endif
+#ifdef VSTATUS
+    DEFSYM(VSTATUS);
+#endif
+#ifdef WERASE
+    DEFSYM(VWERASE);
+#endif
+#ifdef VSWTCH
+    DEFSYM(VSWTCH);
+#endif
+#ifdef VSWTC
+    DEFSYM(VSWTC);
+#endif
+    
     /* extra baudrates.   <= B38400 is defined in termiolib.stub */
 #ifdef B57600
     DEFSYM(B57600);
@@ -295,4 +364,6 @@ void Scm_Init_termios(void)
 #ifdef B230400
     DEFSYM(B230400);
 #endif
+
+#endif /*!__MINGW32__*/
 }
