@@ -129,14 +129,7 @@
 (define (hex-char->number c)
   (if (< c 16)
     c
-    (case (integer->char c)
-      ((#\1) 1) ((#\2) 2) ((#\3) 3)
-      ((#\4) 4) ((#\5) 5) ((#\6) 6)
-      ((#\7) 7) ((#\8) 8) ((#\9) 9)
-      ((#\a #\A) 10) ((#\b #\B) 11)
-      ((#\c #\C) 12) ((#\d #\D) 13)
-      ((#\e #\E) 14) ((#\f #\F) 15)
-      (else 0))))
+    (or (digit->integer (integer->char c) 16) 0)))
 
 (define (number-writer writer . opt-endian)
   (if (pair? opt-endian)
@@ -613,8 +606,8 @@
                  ((procedure? count)
                   (lambda ()
                     (let* ((actual-count (get-count (count-unpack)))
-                           (str (string-incomplete->complete!
-                                 (read-block actual-count)))
+                           (bstr (read-block actual-count))
+                           (str (or (string-incomplete->complete! bstr) bstr))
                            (size (string-size str)))
                       (list
                        (if (< size actual-count)
@@ -622,7 +615,8 @@
                          str)))))
                  (else
                   (lambda ()
-                    (let* ((str (string-incomplete->complete! (read-block count)))
+                    (let* ((bstr (read-block count))
+                           (str (or (string-incomplete->complete! bstr) bstr))
                            (size (string-size str)))
                       (list
                        (if (< size count)
@@ -911,7 +905,8 @@
                  (let ((diff (- var-count (var-len-param))))
                    (var-len-param (+ (var-len-param) diff))
                    (or (port-seek (current-input-port) diff SEEK_CUR)
-                       (read-block diff))))
+                       (read-block diff))
+                   '()))
                #f)))
 
            ;; j   A signed integer value (a Perl internal integer, IV).
