@@ -1,7 +1,7 @@
 ;;;
 ;;; file/util.scm - filesystem utility functions
 ;;;  
-;;;   Copyright (c) 2000-2003 Shiro Kawai, All rights reserved.
+;;;   Copyright (c) 2000-2004 Shiro Kawai, All rights reserved.
 ;;;   
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -30,7 +30,7 @@
 ;;;   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 ;;;   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ;;;  
-;;;  $Id: util.scm,v 1.22 2003/07/05 03:29:11 shirok Exp $
+;;;  $Id: util.scm,v 1.28 2004/11/01 21:53:30 shirok Exp $
 ;;;
 
 ;;; This module provides convenient utility functions to handle
@@ -45,11 +45,12 @@
   (use srfi-13)
   (use util.list)
   (export current-directory directory-list directory-list2 directory-fold
-          home-directory
+          home-directory temporary-directory
           make-directory* create-directory* remove-directory* delete-directory*
           build-path resolve-path expand-path simplify-path
           absolute-path? relative-path? find-file-in-paths
           file-is-readable? file-is-writable? file-is-executable?
+          file-is-symlink?
           file-type file-perm file-mode file-ino file-dev file-rdev file-nlink
           file-uid file-gid file-size file-mtime file-atime file-ctime
           file-eq? file-eqv? file-equal? file-device=?
@@ -57,7 +58,7 @@
           file-atime=? file-atime<? file-atime<=? file-atime>? file-atime>=?
           file-ctime=? file-ctime<? file-ctime<=? file-ctime>? file-ctime>=?
           touch-file copy-file move-file
-          ;;file->string file->string-list file->list file->sexp-list
+          file->string file->string-list file->list file->sexp-list
           ))
 (select-module file.util)
 
@@ -85,6 +86,9 @@
                           (else (error "bad user" user)))))
       (slot-ref ent 'dir))))
 
+(define (temporary-directory)
+  "/tmp")  ;; would be more smarter
+
 ;; utility for directory-list and directory-list2
 (define (%directory-filter dir pred filter-add-path?)
   (if filter-add-path?
@@ -94,7 +98,7 @@
 (define (%directory-filter-compose opts)
   (apply every-pred
          (cond-list ((get-keyword :children? opts #f)
-                     (lambda (e) (not (member e '("." "..")))))
+                     (lambda (e) (not (member (sys-basename e) '("." "..")))))
                     ((get-keyword :filter opts #f)))))
 
 ;; directory-list DIR &keyword ADD-PATH? FILTER-ADD-PATH? CHILDREN? FILTER
@@ -158,7 +162,8 @@
                       d dir))
             (sys-mkdir p mode)
             )))
-    (rec dir)))
+    ;; some platform complains the last "/"
+    (rec (string-trim-right dir #[/]))))
 
 ;; synonym
 (define create-directory* make-directory*)
@@ -190,9 +195,12 @@
                     ((string-prefix? "/" p)
                      (error "can't append absolute path after other path" p))
                     (else p)))
-          (rec (if (string-suffix? "/" base)
-                   (string-append base component)
-                   (string-append base "/" component))
+          (rec (cond ((string-null? base) component)
+                     ((string-null? component) base)
+                     ((string-suffix? "/" base)
+                      (string-append base component))
+                     (else
+                      (string-append base "/" component)))
                (cdr components)))))
   (rec base-path components))
 
@@ -279,6 +287,10 @@
 (define (file-is-writable? path) (sys-access path |W_OK|))
 (define (file-is-executable? path) (sys-access path |X_OK|))
 
+(define (file-is-symlink? path)
+  (and (file-exists? path)
+       (eq? (slot-ref (sys-lstat path) 'type) 'symlink)))
+
 ;; compares two files are identical, in the sense that:
 ;;  file-eq?  - two files (or directories) are the same entity. symbolic
 ;;              links are not followed.   The files must exist.
@@ -489,21 +501,18 @@
 ;; move-directory
 
 ;; file->string, file->list, file->string-list, file->sexp-list
-;; shortcuts of port->string, port->list, port->string-list adn port->sexp-list
+;; shortcuts of port->string etc.
 
-;; NB: this doesn't work well with gauche.charconv.  Redefinition of
-;; call-with-input-file isn't reflected to these definitions.
+(define (file->string file . opts)
+  (apply call-with-input-file file port->string opts))
 
-;(define (file->string file . opts)
-;  (apply call-with-input-file file port->string opts))
+(define (file->list reader file . opts)
+  (apply call-with-input-file file (cut port->list reader <>) opts))
 
-;(define (file->list reader file . opts)
-;  (apply call-with-input-file file (pa$ port->list reader) opts))
+(define (file->string-list file . opts)
+  (apply call-with-input-file file (cut port->list read-line <>) opts))
 
-;(define (file->string-list file . opts)
-;  (apply call-with-input-file file (pa$ port->list read-line) opts))
-
-;(define (file->sexp-list file . opts)
-;  (apply call-with-input-file file (pa$ port->list read) opts))
+(define (file->sexp-list file . opts)
+  (apply call-with-input-file file (cut port->list read <>) opts))
 
 (provide "file/util")
