@@ -1,7 +1,7 @@
 ;;;
 ;;; common-macros.scm - common macros
 ;;;  
-;;;   Copyright (c) 2000-2003 Shiro Kawai, All rights reserved.
+;;;   Copyright (c) 2000-2004 Shiro Kawai, All rights reserved.
 ;;;   
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -30,7 +30,7 @@
 ;;;   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 ;;;   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ;;;  
-;;;  $Id: common-macros.scm,v 1.16 2003/12/19 23:59:23 shirok Exp $
+;;;  $Id: common-macros.scm,v 1.19 2004/08/01 05:41:21 shirok Exp $
 ;;;
 
 ;;; Defines number of useful macros.  This file is loaded by
@@ -48,16 +48,6 @@
 (define-macro (syntax-errorf . args)
   (apply errorf (map unwrap-syntax args)))
 
-;; strip off syntactic information from identifiers in the macro output.
-(define (unwrap-syntax form)
-  (cond
-   ((identifier? form) (identifier->symbol form))
-   ((pair? form) (cons (unwrap-syntax (car form))
-                       (unwrap-syntax (cdr form))))
-   ((vector? form)
-    (list->vector (map unwrap-syntax (vector->list form))))
-   (else form)))
-
 ;;;-------------------------------------------------------------
 ;;; generalized set! family
 
@@ -163,17 +153,19 @@
 ;;;-------------------------------------------------------------
 ;;; bind construct
 
-;; These are experimental --- just giving a try to see if useful
-;; Don't rely on them.  I might change my mind.
-
 (define-syntax let1                     ;single variable bind
   (syntax-rules ()
-    ((_ var exp . body)
+    ((let1 var exp . body)
      (let ((var exp)) . body))))
 
+(define-syntax let/cc                   ;as in PLT
+  (syntax-rules ()
+    ((let/cc var . body)
+     (call/cc (lambda (var) . body)))))
+
 (define-syntax begin0                   ;prog1 in Lisp.
   (syntax-rules ()
-    ((_ exp exp2 ...)
+    ((begin0 exp exp2 ...)
      (receive r exp exp2 ... (apply values r)))))
 
 (define-syntax values-ref               ;nth-value in Lisp
@@ -288,4 +280,35 @@
     ((_ . other)
      (syntax-error "malformed until" (until . other)))))
 
+;;;-------------------------------------------------------------
+;;; guard (srfi-34)
+
+(define-syntax guard
+  (syntax-rules ()
+    ((guard (var . clauses) . body)
+     (with-error-handler
+         (lambda (e)
+           (let ((var e))
+             (%guard-rec var e . clauses)))
+       (lambda () . body)))
+    ))
+
+(define-syntax %guard-rec
+  (syntax-rules (else =>)
+    ((%guard-rec var exc)
+     (raise exc))
+    ((%guard-rec var exc (else . exprs))
+     (begin . exprs))
+    ((%guard-rec var exc (test => proc) . more)
+     (let ((tmp test))
+       (if tmp
+         (proc tmp)
+         (%guard-rec var exc . more))))
+    ((%guard-rec var exc (test . exprs) . more)
+     (if test
+       (begin . exprs)
+       (%guard-rec var exc . more)))
+    ((%guard-rec var exc other . more)
+     (syntax-error "malformed guard clause" other))))
+
 (provide "gauche/common-macros")
