@@ -1,7 +1,7 @@
 ;;;
 ;;; object.scm - object system
 ;;;  
-;;;   Copyright (c) 2000-2003 Shiro Kawai, All rights reserved.
+;;;   Copyright (c) 2000-2004 Shiro Kawai, All rights reserved.
 ;;;   
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -30,7 +30,7 @@
 ;;;   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 ;;;   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ;;;  
-;;;  $Id: object.scm,v 1.51 2003/12/14 00:52:15 shirok Exp $
+;;;  $Id: object.scm,v 1.55 2004/08/01 05:41:21 shirok Exp $
 ;;;
 
 ;; This module is not meant to be `use'd.   It is just to hide
@@ -69,13 +69,15 @@
 ;; Generic function
 ;;
 
-(define-macro (define-generic name)
+(define-macro (define-generic name . opts)
   (receive (true-name getter-name) (%check-setter-name name)
-    (if getter-name
+    (let ((class (get-keyword :class opts <generic>))
+          (other (delete-keyword :class opts)))
+      (if getter-name
         `(begin
-           (define ,true-name (make <generic> :name ',true-name))
+           (define ,true-name (make ,class :name ',true-name ,@other))
            (set! (setter ,getter-name) ,true-name))
-        `(define ,true-name (make <generic> :name ',true-name)))))
+        `(define ,true-name (make ,class :name ',true-name ,@other))))))
 
 ;; allow (setter name) type declaration
 (define (%check-setter-name name)
@@ -526,6 +528,13 @@
 (define-method (setter ref) ((obj <top>) (slot <symbol>) value)
   (slot-set! obj slot value))
 
+(define-method ref ((obj <hash-table>) key)
+  (hash-table-get obj key))
+(define-method ref ((obj <hash-table>) key fallback)
+  (hash-table-get obj key fallback))
+(define-method (setter ref) ((obj <hash-table>) key value)
+  (hash-table-put! obj key value))
+
 ;;----------------------------------------------------------------
 ;; Generalized application hooks
 ;;  (should this be in separate file, e.g. apply.scm?)
