@@ -30,7 +30,7 @@
  *   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  *   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- *  $Id: bignum.c,v 1.55 2004/01/28 08:36:26 shirok Exp $
+ *  $Id: bignum.c,v 1.59 2004/11/05 10:33:37 shirok Exp $
  */
 
 /* Bignum library.  Not optimized well yet---I think bignum performance
@@ -73,6 +73,7 @@ char *alloca ();
 # endif
 #endif
 
+#include <stdlib.h>
 #include <math.h>
 #include <limits.h>
 #define LIBGAUCHE_BODY
@@ -245,56 +246,67 @@ ScmObj Scm_NormalizeBignum(ScmBignum *b)
 }
 
 /* b must be normalized.  */
-long Scm_BignumToSI(ScmBignum *b, int clamphi, int clamplo)
+long Scm_BignumToSI(ScmBignum *b, int clamp, int *oor)
 {
+    if (clamp == SCM_CLAMP_NONE && oor != NULL) *oor = FALSE;
     if (b->sign >= 0) {
         if (b->values[0] > LONG_MAX || b->size >= 2) {
-            if (clamphi) return LONG_MAX;
+            if (clamp & SCM_CLAMP_HI) return LONG_MAX;
             else goto err;
         } else {
             return (long)b->values[0];
         }
     } else {
         if (b->values[0] > (u_long)LONG_MAX+1 || b->size >= 2) {
-            if (clamplo) return LONG_MIN;
+            if (clamp & SCM_CLAMP_LO) return LONG_MIN;
             else goto err;
         } else {
             return -(long)b->values[0];
         }
     }
   err:
-    Scm_Error("argument out of range: %S", SCM_OBJ(b));
-    return 0; /*dummy*/
+    if (clamp == SCM_CLAMP_NONE && oor != NULL) {
+        *oor = TRUE;
+    } else {
+        Scm_Error("argument out of range: %S", SCM_OBJ(b));
+    }
+    return 0;
 }
 
 /* b must be normalized. */
-u_long Scm_BignumToUI(ScmBignum *b, int clamphi, int clamplo)
+u_long Scm_BignumToUI(ScmBignum *b, int clamp, int *oor)
 {
+    if (clamp == SCM_CLAMP_NONE && oor != NULL) *oor = FALSE;
     if (b->sign >= 0) {
         if (b->size >= 2) {
-            if (clamphi) return SCM_ULONG_MAX;
+            if (clamp & SCM_CLAMP_HI) return SCM_ULONG_MAX;
             else goto err;
         } else {
             return b->values[0];
         }
     } else {
-        if (clamplo) return 0;
+        if (clamp & SCM_CLAMP_LO) return 0;
         else goto err;
     }
   err:
-    Scm_Error("argument out of range: %S", SCM_OBJ(b));
-    return 0; /*dummy*/
+    if (clamp == SCM_CLAMP_NONE && oor != NULL) {
+        *oor = TRUE;
+    } else {
+        Scm_Error("argument out of range: %S", SCM_OBJ(b));
+    }
+    return 0;
 }
 
 #if SIZEOF_LONG == 4
 /* we need special routines for int64 */
-ScmInt64 Scm_BignumToSI64(ScmBignum *b, int clamphi, int clamplo)
+ScmInt64 Scm_BignumToSI64(ScmBignum *b, int clamp, int *oor)
 {
 #if SCM_EMULATE_INT64
     ScmInt64 r = {0, 0};
+    if (clamp == SCM_CLAMP_NONE && oor != NULL) *oor = FALSE;
     if (b->sign > 0) {
         if (b->size > 2 || b->values[1] > LONG_MAX) {
-            if (!clamphi) goto err;
+            if (!(clamp & SCM_CLAMP_HI)) goto err;
             SCM_SET_INT64_MAX(r);
         } else {
             r.lo = b->values[0];
@@ -302,7 +314,7 @@ ScmInt64 Scm_BignumToSI64(ScmBignum *b, int clamphi, int clamplo)
         }
     } else if (b->sign < 0) {
         if (b->size > 2 || b->values[1] > (u_long)LONG_MAX + 1) {
-            if (!clamplo) goto err;
+            if (!(clamp&SCM_CLAMP_LO)) goto err;
             SCM_SET_INT64_MIN(r);
         } else {
             b = SCM_BIGNUM(Scm_BignumComplement(b));
@@ -312,74 +324,80 @@ ScmInt64 Scm_BignumToSI64(ScmBignum *b, int clamphi, int clamplo)
         }
     }
     return r;
-  err:
-    Scm_Error("argument out of range: %S", SCM_OBJ(b));
-    return r; /*dummy*/
 #else  /*!SCM_EMULATE_INT64*/
     int64_t r = 0;
+    if (clamp == SCM_CLAMP_NONE && oor != NULL) *oor = FALSE;
     if (b->sign > 0) {
-        if (b->size > 2 || b->values[1] > LONG_MAX) {
-            if (!clamphi) goto err;
+        if (b->size == 1) {
+            r = b->values[0];
+        } else if (b->size > 2 || b->values[1] > LONG_MAX) {
+            if (!(clamp & SCM_CLAMP_HI)) goto err;
             SCM_SET_INT64_MAX(r);
         } else {
             r = ((int64_t)b->values[1] << 32) + (uint64_t)b->values[0];
         }
     } else { /* b->sign < 0 */
-        if (b->size > 2
-            || ((b->values[1] > (u_long)LONG_MAX)
-                && (b->values[0] > 0))) {
-            if (!clamplo) goto err;
+        if (b->size == 1) {
+            r = -(int64_t)b->values[0];
+        } else if (b->size > 2 || (b->values[1] > LONG_MAX && b->values[0] > 0)) {
+            if (!(clamp&SCM_CLAMP_LO)) goto err;
             SCM_SET_INT64_MIN(r);
         } else {
-            b = SCM_BIGNUM(Scm_BignumComplement(b));
-            r = ((int64_t)b->values[1] << 32) + (uint64_t)b->values[0];
+            r = -(((int64_t)b->values[1] << 32) + (uint64_t)b->values[0]);
         }
     }
     return r;
-  err:
-    Scm_Error("argument out of range: %S", SCM_OBJ(b));
-    return 0; /*dummy*/
 #endif /*!SCM_EMULATE_INT64*/
+  err:
+    if (clamp == SCM_CLAMP_NONE && oor != NULL) {
+        *oor = TRUE;
+    } else {
+        Scm_Error("argument out of range: %S", SCM_OBJ(b));
+    }
+    return r;
 }
 
-ScmUInt64 Scm_BignumToUI64(ScmBignum *b, int clamphi, int clamplo)
+ScmUInt64 Scm_BignumToUI64(ScmBignum *b, int clamp, int *oor)
 {
 #if SCM_EMULATE_INT64
     ScmInt64 r = {0, 0};
+    if (clamp == SCM_CLAMP_NONE && oor != NULL) *oor = FALSE;
     if (b->sign > 0) {
         if (b->size > 2) {
-            if (!clamphi) goto err;
+            if (!(clamp&SCM_CLAMP_HI)) goto err;
             SCM_SET_UINT64_MAX(r);
         } else {
             r.lo = b->values[0];
             if (b->size == 2) r.hi = b->values[1];
         }
     } else if (b->sign < 0) {
-        if (!clamplo) goto err;
+        if (!(clamp&SCM_CLAMP_LO)) goto err;
     }
     return r;
-  err:
-    Scm_Error("argument out of range: %S", SCM_OBJ(b));
-    return r; /*dummy*/
 #else  /*!SCM_EMULATE_INT64*/
     uint64_t r = 0;
+    if (clamp == SCM_CLAMP_NONE && oor != NULL) *oor = FALSE;
     if (b->sign > 0) {
         if (b->size > 2) {
-            if (!clamphi) goto err;
-            r = SCM_SET_UINT64_MAX(r);
+            if (!(clamp&SCM_CLAMP_HI)) goto err;
+            SCM_SET_UINT64_MAX(r);
         } else if (b->size == 2) {
-            r = ((uint64_t)b->values[1] << 32) + (uint64_t)b->values[0];
+            r = (((uint64_t)b->values[1]) << 32) + (uint64_t)b->values[0];
         } else {
             r = (uint64_t)b->values[0];
         }
     } else { /* b->sign < 0 */
-        if (!clamplo) goto err;
+        if (!(clamp&SCM_CLAMP_LO)) goto err;
     }
     return r;
-  err:
-    Scm_Error("argument out of range: %S", SCM_OBJ(b));
-    return 0; /*dummy*/
 #endif /*!SCM_EMULATE_INT64*/
+  err:
+    if (clamp == SCM_CLAMP_NONE && oor != NULL) {
+        *oor = TRUE;
+    } else {
+        Scm_Error("argument out of range: %S", SCM_OBJ(b));
+    }
+    return r;
 }
 #endif /* SIZEOF_LONG == 4 */
 
@@ -957,10 +975,10 @@ static ScmBignum *bignum_gdiv(ScmBignum *dividend, ScmBignum *divisor,
     vn_2 = DIGIT(v, n-2);
 #undef DIV_DEBUG
 #ifdef DIV_DEBUG
-    printf("shift=%d, n=%d, m=%d\n", d, n, m);
-    printf("u="); Scm_DumpBignum(u, SCM_CUROUT); printf("\n");
-    printf("v="); Scm_DumpBignum(v, SCM_CUROUT); printf("\n");
-    printf("vn_1=%08lx, vn_2=%08lx\n", vn_1, vn_2);
+    Scm_Printf(SCM_CUROUT, "shift=%d, n=%d, m=%d\n", d, n, m);
+    Scm_Printf(SCM_CUROUT, "u="); Scm_DumpBignum(u, SCM_CUROUT);
+    Scm_Printf(SCM_CUROUT, "\nv="); Scm_DumpBignum(v, SCM_CUROUT);
+    Scm_Printf(SCM_CUROUT, "\nvn_1=%08lx, vn_2=%08lx\n", vn_1, vn_2);
 #endif
 
     for (j = m; j >= 0; j--) {
@@ -968,14 +986,16 @@ static ScmBignum *bignum_gdiv(ScmBignum *dividend, ScmBignum *divisor,
         u_long qq = uu/vn_1;
         u_long rr = uu%vn_1;
 #ifdef DIV_DEBUG
-        printf("loop on j=%d, uu=%08lx, qq=%08lx, rr=%08lx\n", j, uu, qq, rr);
+        Scm_Printf(SCM_CUROUT, "loop on j=%d, uu=%08lx, qq=%08lx, rr=%08lx\n",
+                   j, uu, qq, rr);
 #endif
-        if (qq == HALF_WORD) { qq--; rr += vn_1; }
+        while (qq >= HALF_WORD) { qq--; rr += vn_1; }
         while ((qq*vn_2 > (rr<<HALF_BITS)+DIGIT(u, j+n-2)) && (rr < HALF_WORD)) {
             qq--; rr += vn_1;
         }
 #ifdef DIV_DEBUG
-        printf("estimate uu=%08lx, qq=%08lx, rr=%08lx\n", uu, qq, rr);
+        Scm_Printf(SCM_CUROUT, "estimate uu=%08lx, qq=%08lx, rr=%08lx\n",
+                   uu, qq, rr);
 #endif
         cy = 0;
         for (k = 0; k < n; k++) {
@@ -986,8 +1006,9 @@ static ScmBignum *bignum_gdiv(ScmBignum *dividend, ScmBignum *divisor,
             SETDIGIT2(u, j+k, uj2);
         }
 #ifdef DIV_DEBUG
-        printf("subtract cy = %d, ", cy);
-        printf("u="); Scm_DumpBignum(u, SCM_CUROUT); printf("\n");
+        Scm_Printf(SCM_CUROUT, "subtract cy = %d, ", cy);
+        Scm_Printf(SCM_CUROUT, "u="); Scm_DumpBignum(u, SCM_CUROUT);
+        Scm_Printf(SCM_CUROUT, "\n");
 #endif
         if (cy) {
             qq--;
@@ -1005,8 +1026,9 @@ static ScmBignum *bignum_gdiv(ScmBignum *dividend, ScmBignum *divisor,
     }
     bignum_rshift(u, u, d);
 #ifdef DIV_DEBUG
-    printf("quot q="); Scm_DumpBignum(quotient, SCM_CUROUT); printf("\n");
-    printf("rem  u="); Scm_DumpBignum(u, SCM_CUROUT); printf("\n");
+    Scm_Printf(SCM_CUROUT, "quot q="); Scm_DumpBignum(quotient, SCM_CUROUT);
+    Scm_Printf(SCM_CUROUT, "\nrem  u="); Scm_DumpBignum(u, SCM_CUROUT);
+    Scm_Printf(SCM_CUROUT, "\n");
 #endif
     return u;
 }
