@@ -1,7 +1,7 @@
 /*
  * core.c - core kernel interface
  *
- *   Copyright (c) 2000-2003 Shiro Kawai, All rights reserved.
+ *   Copyright (c) 2000-2004 Shiro Kawai, All rights reserved.
  * 
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -30,7 +30,7 @@
  *   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  *   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- *  $Id: core.c,v 1.53 2003/12/16 09:50:46 shirok Exp $
+ *  $Id: core.c,v 1.61 2004/11/26 01:45:39 shirok Exp $
  */
 
 #include <stdlib.h>
@@ -72,6 +72,7 @@ extern void Scm__InitSignal(void);
 extern void Scm__InitSystem(void);
 extern void Scm__InitVM(void);
 extern void Scm__InitRepl(void);
+extern void Scm__InitParameter(void);
 extern void Scm__InitAutoloads(void);
 
 extern void Scm_Init_stdlib(ScmModule *);
@@ -79,6 +80,9 @@ extern void Scm_Init_extlib(ScmModule *);
 extern void Scm_Init_syslib(ScmModule *);
 extern void Scm_Init_moplib(ScmModule *);
 
+static void finalizable(void);
+
+
 #ifdef GAUCHE_USE_PTHREADS
 /* a trick to make sure the gc thread object is linked */
 static int (*ptr_pthread_create)(void) = NULL;
@@ -96,7 +100,12 @@ void Scm_Init(const char *signature)
        already initialized, so we call it here just in case. */
     GC_init();
 
+    /* Set up GC parameters.  We need to call finalizers at the safe
+       point of VM loop, so we disable auto finalizer invocation, and
+       ask GC to call us back when finalizers are queued. */
     GC_oom_fn = oom_handler;
+    GC_finalize_on_demand = TRUE;
+    GC_finalizer_notifier = finalizable;
 
     /* Initialize components.  The order is important, for some components
        rely on the other components to be initialized. */
@@ -112,6 +121,7 @@ void Scm_Init(const char *signature)
     Scm__InitWrite();
     Scm__InitVM();
     Scm__InitCompiler();
+    Scm__InitParameter();
     Scm__InitMacro();
     Scm__InitLoad();
     Scm__InitRegexp();
@@ -153,6 +163,27 @@ void Scm_RegisterFinalizer(ScmObj z, ScmFinalizerProc finalizer, void *data)
                                    data, &ofn, &ocd);
 }
 
+void Scm_UnregisterFinalizer(ScmObj z)
+{
+    GC_finalization_proc ofn; GC_PTR ocd;
+    GC_REGISTER_FINALIZER_NO_ORDER(z, (GC_finalization_proc)NULL, NULL,
+                                   &ofn, &ocd);
+}
+
+/* GC calls this back when finalizers are queued */
+void finalizable(void)
+{
+    ScmVM *vm = Scm_VM();
+    vm->queueNotEmpty |= SCM_VM_FINQ_MASK;
+}
+
+/* Called from VM loop.  Queue is not empty. */
+ScmObj Scm_VMFinalizerRun(ScmVM *vm)
+{
+    GC_invoke_finalizers();
+    vm->queueNotEmpty &= ~SCM_VM_FINQ_MASK;
+}
+
 /*
  * Program termination
  */
@@ -169,6 +200,7 @@ void Scm_Exit(int code)
         Scm_Apply(SCM_CDAR(hp), SCM_NIL);
     }
     Scm_FlushAllPorts(TRUE);
+
     exit(code);
 }
 
@@ -193,6 +225,10 @@ void Scm_Abort(const char *msg)
 
 /*
  * Inspect the configuration
+ * For MinGW32, we don't know where things will be installed at
+ * the compile time, so we don't use configure-variable GAUCHE_LIB_DIR etc.
+ * Instead, we query the directory of DLL and calculate the paths
+ *
  */
 
 const char *Scm_HostArchitecture(void)
@@ -200,6 +236,115 @@ const char *Scm_HostArchitecture(void)
     return GAUCHE_ARCH;
 }
 
+#ifdef __MINGW32__
+
+ScmObj get_install_dir(void)
+{
+    static ScmObj dir = SCM_FALSE;
+    if (SCM_FALSEP(dir)) {
+	HMODULE mod;
+	DWORD r;
+	char path[MAX_PATH];
+
+	mod = GetModuleHandle("libgauche.dll");
+	if (mod == NULL) {
+	    Scm_Error("GetModuleHandle failed");
+	}
+	r = GetModuleFileName(mod, path, MAX_PATH);
+	if (r == 0) {
+	    Scm_Error("GetModuleFileName failed");
+	}
+	/* remove \libgauche.dll */
+	if (!PathRemoveFileSpec(path)) {
+	    Scm_Error("PathRemoveFileSpec failed on %s", path);
+	}
+	/* remobe \bin */
+	if (!PathRemoveFileSpec(path)) {
+	    Scm_Error("PathRemoveFileSpec failed on %s", path);
+	}
+	dir = SCM_MAKE_STR_COPYING(path);
+    }
+    return dir;
+}
+
+ScmObj Scm_LibraryDirectory(void)
+{
+    static ScmObj dir = SCM_FALSE;
+    if (SCM_FALSEP(dir)) {
+        dir = Scm_StringAppendC(SCM_STRING(get_install_dir()),
+                                "\\share\\gauche\\"GAUCHE_VERSION"\\lib",
+                                -1, -1);
+    }
+    return dir;
+}
+
+ScmObj Scm_ArchitectureDirectory(void)
+{
+    static ScmObj dir = SCM_FALSE;
+    if (SCM_FALSEP(dir)) {
+        dir = Scm_StringAppendC(SCM_STRING(get_install_dir()),
+                                "\\lib\\gauche\\"GAUCHE_VERSION"\\"GAUCHE_ARCH,
+                                -1, -1);
+    }
+    return dir;
+}
+
+ScmObj Scm_SiteLibraryDirectory(void)
+{
+    static ScmObj dir = SCM_FALSE;
+    if (SCM_FALSEP(dir)) {
+        dir = Scm_StringAppendC(SCM_STRING(get_install_dir()),
+                                "\\share\\gauche\\site\\lib",
+                                -1, -1);
+    }
+    return dir;
+}
+
+ScmObj Scm_SiteArchitectureDirectory(void)
+{
+    static ScmObj dir = SCM_FALSE;
+    if (SCM_FALSEP(dir)) {
+        dir = Scm_StringAppendC(SCM_STRING(get_install_dir()),
+                                "\\lib\\gauche\\site\\"GAUCHE_VERSION"\\"GAUCHE_ARCH,
+                                -1, -1);
+    }
+    return dir;
+}
+
+#else /* !__MINGW32__ */
+
+#define DEFSTR(n, s) \
+    static SCM_DEFINE_STRING_CONST(n, s, sizeof(s)-1, sizeof(s)-1)
+
+DEFSTR(libdir,      GAUCHE_LIB_DIR);
+DEFSTR(archdir,     GAUCHE_ARCH_DIR);
+DEFSTR(sitelibdir,  GAUCHE_SITE_LIB_DIR);
+DEFSTR(sitearchdir, GAUCHE_SITE_ARCH_DIR);
+
+ScmObj Scm_LibraryDirectory(void)
+{
+    return SCM_OBJ(&libdir);
+}
+
+ScmObj Scm_ArchitectureDirectory(void)
+{
+    return SCM_OBJ(&archdir);
+}
+
+ScmObj Scm_SiteLibraryDirectory(void)
+{
+    return SCM_OBJ(&sitelibdir);
+}
+
+ScmObj Scm_SiteArchitectureDirectory(void)
+{
+    return SCM_OBJ(&sitearchdir);
+}
+
+#undef DEFSTR
+
+#endif /* !__MINGW32__ */
+
 /*
  * Useful routine for debugging, to check if an object is inadvertently
  * collected.
