@@ -30,7 +30,7 @@
  *   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  *   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- *  $Id: vm.h,v 1.88 2004/02/02 10:43:37 shirok Exp $
+ *  $Id: vm.h,v 1.98 2004/11/29 21:54:23 shirok Exp $
  */
 
 #ifndef GAUCHE_VM_H
@@ -45,8 +45,26 @@
 /* Signal queue size */
 #define SCM_VM_SIGQ_SIZE       32
 
+/* Finalizer queue size */
+#define SCM_VM_FINQ_SIZE       32
+
+/* QueueNotEmpty flag */
+#define SCM_VM_SIGQ_MASK       1
+#define SCM_VM_FINQ_MASK       2
+
 #define SCM_PCTYPE ScmObj
 
+/*
+ * Compiled code packet
+ */
+
+typedef struct ScmCompiledCodeRec {
+    ScmObj *code;               /* code vector */
+    ScmObj *constants;          /* constant vector */
+    int maxstack;               /* maximum runtime stack depth */
+    ScmObj info;                /* debug info */
+} ScmCompiledCode;
+
 /*
  * Environment frame
  *
@@ -176,14 +194,57 @@ typedef struct ScmVMParameterTableRec {
     int numParameters;
     int numAllocated;
     ScmObj *vector;
+    int *ids;
 } ScmVMParameterTable;
 
 SCM_EXTERN void Scm_ParameterTableInit(ScmVMParameterTable *table,
                                        ScmVM *base);
 
-SCM_EXTERN int Scm_MakeParameterSlot(ScmVM *vm);
-SCM_EXTERN ScmObj Scm_ParameterRef(ScmVM *vm, int index);
-SCM_EXTERN ScmObj Scm_ParameterSet(ScmVM *vm, int index, ScmObj value);
+SCM_EXTERN int Scm_MakeParameterSlot(ScmVM *vm, int *newid);
+SCM_EXTERN ScmObj Scm_ParameterRef(ScmVM *vm, int index, int id);
+SCM_EXTERN ScmObj Scm_ParameterSet(ScmVM *vm, int index, int id, ScmObj value);
+
+/*
+ * Signal queue
+ *
+ *  Gauche installs singnal handlers that simply queues the signal.
+ *  See signal.c for the implementaiton.
+ *  Signal queue is a fixed size, since we can't really extend it
+ *  within the system's signal handler.  For the time being, overflowed
+ *  signals are simply discarded.
+ */
+
+typedef struct ScmSignalQueueRec {
+    int queue[SCM_VM_SIGQ_SIZE];/* Ring buffer for pending signals */
+    unsigned int head;     /* points to the queue head */
+    unsigned int tail;     /* points to the queue tail */
+    unsigned int overflow; /* flag to indicate queue overflow */
+    ScmObj pending;        /* pending signal handlers */
+} ScmSignalQueue;
+
+SCM_EXTERN void Scm_SignalQueueInit(ScmSignalQueue* q);
+
+/* NB: Obsoleted macro, but kept for backward compatibility.
+   Better API should be provided in future. */
+#define SCM_SIGPENDING(vm) \
+    ((vm)->queueNotEmpty&SCM_VM_SIGQ_MASK)
+
+#define SCM_SIGCHECK(vm) \
+    do { if (vm->queueNotEmpty&SCM_VM_SIGQ_MASK) Scm_SigCheck(vm); } while (0)
+
+SCM_EXTERN void   Scm_SigCheck(ScmVM *vm);
+
+/*
+ * Finalizers
+ *
+ *  Finalizers are queued inside GC.  We disable automatic finalizer
+ *  invocation of GC and only set the flag on VM when finalizers are
+ *  queued.  VM loop check the flag and calls Scm_VMFinalizerRun()
+ *  to run the finalizers.  (If VM is not running, C library must
+ *  call it explicitly to run finalizers).
+ */
+
+SCM_EXTERN ScmObj Scm_VMFinalizerRun(ScmVM *vm);
 
 /*
  * VM structure
@@ -226,6 +287,7 @@ struct ScmVMRec {
                                    "C stack rewinding" below. */
     unsigned int runtimeFlags;  /* Runtime flags */
     unsigned int compilerFlags; /* Compiler flags */
+    unsigned int queueNotEmpty; /* Bitmask if sigq or finq is not empty */
 
     ScmPort *curin;             /* current input port */
     ScmPort *curout;            /* current output port */
@@ -272,11 +334,7 @@ struct ScmVMRec {
     ScmObj load_port;           /* current port from which we are loading */
 
     /* Signal information */
-    int sigQueue[SCM_VM_SIGQ_SIZE];/* Ring buffer for pending signals */
-    unsigned int sigQueueHead;  /* points to the queue head */
-    unsigned int sigQueueTail;  /* points to the queue tail */
-    unsigned int sigOverflow;   /* flag to indicate queue overflow */
-    ScmObj sigPending;          /* pending signal handlers */
+    ScmSignalQueue sigq;
     sigset_t sigMask;           /* current signal mask */
 };
 
@@ -383,7 +441,7 @@ SCM_EXTERN ScmObj Scm_VMInsnInspect(ScmObj obj);
        cstack.prev = Scm_VM()->cstack;          \
        cstack.cont = NULL;                      \
        Scm_VM()->cstack = &cstack;              \
-       if (sigsetjmp(cstack.jbuf, TRUE) == 0) {
+       if (sigsetjmp(cstack.jbuf, FALSE) == 0) {
            
 #define SCM_WHEN_ERROR                          \
        } else {
@@ -452,6 +510,10 @@ enum {
     SCM_COMPILE_NORMAL          /* Normal calling sequence. */
 };
 
+typedef struct ScmCompilerPacketRec {
+    ScmObj constants;
+} ScmCompilerContext;
+
 /*
  * Compiler flags
  */
@@ -466,4 +528,14 @@ enum {
 #define SCM_VM_COMPILER_FLAG_SET(vm, flag)    ((vm)->compilerFlags |= (flag))
 #define SCM_VM_COMPILER_FLAG_CLEAR(vm, flag)  ((vm)->compilerFlags &= ~(flag))
 
+/*
+ * Inline assembler stuff
+ *   These are used in Gauche core to inline VM assembly code for
+ *   some built-in procedures.
+ */
+
+SCM_EXTERN ScmObj Scm_MakeInlineAsmForm(ScmObj form, ScmObj insn, ScmObj args);
+SCM_EXTERN ScmObj Scm_SimpleAsmInliner(ScmObj subr, ScmObj form, ScmObj env,
+                                       void *data);
+
 #endif /* GAUCHE_VM_H */
