@@ -30,7 +30,7 @@
  *   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  *   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- *  $Id: load.c,v 1.82 2004/01/18 12:07:31 shirok Exp $
+ *  $Id: load.c,v 1.94 2004/11/21 12:46:59 shirok Exp $
  */
 
 #include <stdlib.h>
@@ -57,7 +57,8 @@ static struct {
     /* Load path list */
     ScmGloc *load_path_rec;     /* *load-path*         */
     ScmGloc *dynload_path_rec;  /* *dynamic-load-path* */
-    ScmGloc *load_suffixes_rec; /* *load-suffixes* */
+    ScmGloc *load_suffixes_rec; /* *load-suffixes*     */
+    ScmGloc *cond_features_rec; /* *cond-features*     */
     ScmInternalMutex path_mutex;
 
     /* Provided features */
@@ -74,13 +75,14 @@ static struct {
     ScmObj dso_suffixes;
     ScmObj dso_list;              /* List of dynamically loaded objects. */
     ScmInternalMutex dso_mutex;
-} ldinfo = { NULL, };
+} ldinfo = { (ScmGloc*)&ldinfo, };  /* trick to put ldinfo in .data section */
 
 /* keywords used for load and load-from-port surbs */
 static ScmObj key_paths              = SCM_UNBOUND;
 static ScmObj key_error_if_not_found = SCM_UNBOUND;
 static ScmObj key_environment        = SCM_UNBOUND;
 static ScmObj key_macro              = SCM_UNBOUND;
+static ScmObj key_ignore_coding      = SCM_UNBOUND;
 
 /*--------------------------------------------------------------------
  * Scm_LoadFromPort
@@ -94,6 +96,10 @@ static ScmObj key_macro              = SCM_UNBOUND;
  *   No matter how the load terminates, either normal or abnormal,
  *   the port is closed, and the current module is restored to the
  *   one when load is called.
+ *
+ *   FLAGS argument is ignored for now, but reserved for future
+ *   extension.  SCM_LOAD_QUIET_NOFILE and SCM_LOAD_IGNORE_CODING
+ *   won't have any effect for LoadFromPort; see Scm_Load below.
  */
 
 struct load_packet {
@@ -137,13 +143,15 @@ static ScmObj load_body(ScmObj *args, int nargs, void *data)
     return load_cc(SCM_NIL, &data);
 }
 
-ScmObj Scm_VMLoadFromPort(ScmPort *port, ScmObj next_paths, ScmObj env)
+ScmObj Scm_VMLoadFromPort(ScmPort *port, ScmObj next_paths,
+                          ScmObj env, int flags)
 {
     struct load_packet *p;
     ScmObj port_info;
     ScmVM *vm = Scm_VM();
     ScmModule *module = vm->module;
-    
+
+    /* Sanity check */
     if (!SCM_IPORTP(port))
         Scm_Error("input port required, but got: %S", port);
     if (SCM_PORT_CLOSED_P(port))
@@ -185,13 +193,15 @@ static ScmObj load_from_port(ScmObj *args, int argc, void *data)
 {
     ScmPort *port;
     ScmObj paths, env;
+    int flags = 0;
+    
     if (!SCM_IPORTP(args[0])) {
         Scm_Error("input port required, but got %S", args[0]);
     }
     port = SCM_PORT(args[0]);
     paths = Scm_GetKeyword(key_paths, args[1], SCM_FALSE);
     env   = Scm_GetKeyword(key_environment, args[1], SCM_FALSE);
-    return Scm_VMLoadFromPort(port, paths, env);
+    return Scm_VMLoadFromPort(port, paths, env, flags);
 }
 
 static SCM_DEFINE_STRING_CONST(load_from_port_NAME, "load-from-port", 14, 14);
@@ -199,7 +209,7 @@ static SCM_DEFINE_SUBR(load_from_port_STUB, 1, 1,
                        SCM_OBJ(&load_from_port_NAME), load_from_port,
                        NULL, NULL);
 
-void Scm_LoadFromPort(ScmPort *port)
+void Scm_LoadFromPort(ScmPort *port, int flags)
 {
     Scm_Apply(SCM_OBJ(&load_from_port_STUB), SCM_LIST1(SCM_OBJ(port)));
 }
@@ -246,13 +256,13 @@ static ScmObj try_suffixes(ScmObj base, ScmObj suffixes)
 }
 
 ScmObj Scm_FindFile(ScmString *filename, ScmObj *paths,
-                    ScmObj suffixes, int error_if_not_found)
+                    ScmObj suffixes, int flags)
 {
     int size = SCM_STRING_LENGTH(filename);
     const char *ptr = SCM_STRING_START(filename);
     int use_load_paths = TRUE;
     ScmObj file = SCM_OBJ(filename), fpath = SCM_FALSE;
-    
+
     if (size == 0) Scm_Error("bad filename to load: \"\"");
     if (*ptr == '~') {
         file = Scm_NormalizePathname(filename, SCM_PATH_EXPAND);
@@ -260,10 +270,10 @@ ScmObj Scm_FindFile(ScmString *filename, ScmObj *paths,
     } else if (*ptr == '/'
                || (*ptr == '.' && *(ptr+1) == '/')
                || (*ptr == '.' && *(ptr+1) == '.' && *(ptr+2) == '/')
-#ifdef __CYGWIN__
+#if defined(__CYGWIN__) || defined(__MINGW32__)
 	       /* support for wicked legacy DOS drive letter */
 	       || (isalpha(*ptr) && *(ptr+1) == ':')
-#endif /* __CYGWIN__ */
+#endif /* __CYGWIN__ || __MINGW32__ */
 	       ) {
         use_load_paths = FALSE;
     }
@@ -282,7 +292,7 @@ ScmObj Scm_FindFile(ScmString *filename, ScmObj *paths,
         if (SCM_PAIRP(lpath)) {
             *paths = SCM_CDR(lpath);
             return SCM_OBJ(fpath);
-        } else if (error_if_not_found) {
+        } else if (!(flags&SCM_LOAD_QUIET_NOFILE)) {
             Scm_Error("cannot find file %S in *load-path* %S", file, *paths);
         } else {
             *paths = SCM_NIL;
@@ -291,71 +301,101 @@ ScmObj Scm_FindFile(ScmString *filename, ScmObj *paths,
         *paths = SCM_NIL;
         fpath = try_suffixes(file, suffixes);
         if (!SCM_FALSEP(fpath)) return fpath;
-        if (error_if_not_found) {
+        if (!(flags&SCM_LOAD_QUIET_NOFILE)) {
             Scm_Error("cannot find file %S to load", file);
         }
     }
     return SCM_FALSE;
 }
 
-/*
- * Load
+/*---------------------------------------------------------------------
+ * Scm_Load
+ *
+ *  Scheme's load().
+ * 
+ *  filename   - name of the file.  can be sans suffix.
+ *  load_paths - list of pathnames or #f.   If #f, system's load path
+ *               is used.
+ *  env        - a module where the forms are evaluated, or #f.
+ *               If #f, the current module is used.
+ *  flags      - combination of bit flags
+ *               SCM_LOAD_QUIET_NOFILE, SCM_LOAD_IGNORE_CODING
  */
 
 ScmObj Scm_VMLoad(ScmString *filename, ScmObj load_paths,
-                  ScmObj env, int errorp)
+                  ScmObj env, int flags)
 {
     ScmObj port, truename, suffixes;
     ScmVM *vm = Scm_VM();
+    int errorp = !(flags&SCM_LOAD_QUIET_NOFILE);
+    int ignore_coding = flags&SCM_LOAD_IGNORE_CODING;
 
     suffixes = SCM_GLOC_GET(ldinfo.load_suffixes_rec);
     if (!SCM_PAIRP(load_paths)) load_paths = Scm_GetLoadPath();
-    truename = Scm_FindFile(filename, &load_paths, suffixes, errorp);
+    truename = Scm_FindFile(filename, &load_paths, suffixes, flags);
     if (SCM_FALSEP(truename)) return SCM_FALSE;
+
     if (SCM_VM_RUNTIME_FLAG_IS_SET(vm, SCM_LOAD_VERBOSE)) {
         int len = Scm_Length(vm->load_history);
         SCM_PUTZ(";;", 2, SCM_CURERR);
         while (len-- > 0) SCM_PUTC(' ', SCM_CURERR);
         Scm_Printf(SCM_CURERR, "Loading %A...\n", truename);
     }
+
     port = Scm_OpenFilePort(Scm_GetStringConst(SCM_STRING(truename)),
                             O_RDONLY, SCM_PORT_BUFFER_FULL, 0);
     if (SCM_FALSEP(port)) {
         if (errorp) Scm_Error("file %S exists, but couldn't open.", truename);
         else        return SCM_FALSE;
     }
-    return Scm_VMLoadFromPort(SCM_PORT(port), load_paths, env);
+
+    if (!ignore_coding) {
+        port = Scm_MakeCodingAwarePort(SCM_PORT(port));
+    }
+    
+    return Scm_VMLoadFromPort(SCM_PORT(port), load_paths, env, flags);
 }
 
-/* Scheme subr (%load filename &keyword paths error-if-not-found environment) */
+/* Scheme subr (%load filename &keyword paths error-if-not-found
+                                        environment aware-coding) */
 static ScmObj load(ScmObj *args, int argc, void *data)
 {
     ScmString *file;
     ScmObj paths, env;
-    int errorp;
+    int flags = 0;
+    
     if (!SCM_STRINGP(args[0])) {
         Scm_Error("string required, but got %S", args[0]);
     }
     file = SCM_STRING(args[0]);
-    paths  = Scm_GetKeyword(key_paths, args[1], SCM_FALSE);
-    errorp = !SCM_FALSEP(Scm_GetKeyword(key_error_if_not_found, args[1], SCM_TRUE));
-    env    = Scm_GetKeyword(key_environment, args[1], SCM_FALSE);
-    return Scm_VMLoad(file, paths, env, errorp);
+    paths = Scm_GetKeyword(key_paths, args[1], SCM_FALSE);
+    env   = Scm_GetKeyword(key_environment, args[1], SCM_FALSE);
+    if (SCM_FALSEP(Scm_GetKeyword(key_error_if_not_found, args[1], SCM_TRUE)))
+        flags |= SCM_LOAD_QUIET_NOFILE;
+    if (!SCM_FALSEP(Scm_GetKeyword(key_ignore_coding, args[1], SCM_FALSE)))
+        flags |= SCM_LOAD_IGNORE_CODING;
+    return Scm_VMLoad(file, paths, env, flags);
 }
 
 static SCM_DEFINE_STRING_CONST(load_NAME, "load", 4, 4);
 static SCM_DEFINE_SUBR(load_STUB, 1, 1, SCM_OBJ(&load_NAME), load, NULL, NULL);
 
 
-int Scm_Load(const char *cpath, int errorp)
+int Scm_Load(const char *cpath, int flags)
 {
     ScmObj r, f = SCM_MAKE_STR_COPYING(cpath);
-    if (errorp) {
-        r = Scm_Apply(SCM_OBJ(&load_STUB), SCM_LIST1(f));
-    } else {
-        r = Scm_Apply(SCM_OBJ(&load_STUB),
-                      SCM_LIST3(f, key_error_if_not_found, SCM_FALSE));
+    ScmObj options = SCM_NIL;
+    
+    if (!(flags&SCM_LOAD_QUIET_NOFILE)) {
+        options = Scm_Cons(key_error_if_not_found,
+                           Scm_Cons(SCM_FALSE, options));
     }
+    if (flags&SCM_LOAD_IGNORE_CODING) {
+        options = Scm_Cons(key_ignore_coding,
+                           Scm_Cons(SCM_TRUE, options));
+    }
+    
+    r = Scm_Apply(SCM_OBJ(&load_STUB), Scm_Cons(f, options));
     return !SCM_FALSEP(r);
 }
 
@@ -384,9 +424,21 @@ ScmObj Scm_GetDynLoadPath(void)
 static ScmObj break_env_paths(const char *envname)
 {
     const char *e = getenv(envname);
-    if (geteuid() == 0) return SCM_NIL; /* don't trust env when run by root */
-    if (e == NULL) return SCM_NIL;
-    else return Scm_StringSplitByChar(SCM_STRING(SCM_MAKE_STR_COPYING(e)), ':');
+#ifndef __MINGW32__
+    char delim = ':';
+#else  /*__MINGW32__*/
+    char delim = ';';
+#endif /*__MINGW32__*/
+
+    if (e == NULL) {
+	return SCM_NIL;
+    } else if (Scm_IsSugid()) {
+        /* don't trust env when setugid'd */
+        return SCM_NIL;
+    } else {
+	return Scm_StringSplitByChar(SCM_STRING(SCM_MAKE_STR_COPYING(e)),
+				     delim);
+    }
 }
 
 /* Add CPATH to the current list of load path.  The path is
@@ -497,6 +549,8 @@ typedef void (*ScmDynLoadInitFn)(void);
    not much point to avoid dlopen here. */
 #if defined(HAVE_DLOPEN)
 #include "dl_dlopen.c"
+#elif defined(__MINGW32__)
+#include "dl_win.c"
 #else
 #include "dl_dummy.c"
 #endif
@@ -600,7 +654,7 @@ static ScmObj find_so_from_la(ScmString *lafile)
 /* Dynamically load the specified object by FILENAME.
    FILENAME must not contain the system's suffix (.so, for example).
 */
-ScmObj Scm_DynLoad(ScmString *filename, ScmObj initfn, int export)
+ScmObj Scm_DynLoad(ScmString *filename, ScmObj initfn, int export_)
 {
     ScmObj reqname, truename, load_paths = Scm_GetDynLoadPath();
     void *handle;
@@ -789,7 +843,7 @@ ScmObj Scm_Require(ScmObj feature)
     if (!SCM_FALSEP(provided)) return SCM_TRUE;
     SCM_UNWIND_PROTECT {
         filename = Scm_StringAppendC(SCM_STRING(feature), ".scm", 4, 4);
-        Scm_Load(Scm_GetStringConst(SCM_STRING(filename)), TRUE);
+        Scm_Load(Scm_GetStringConst(SCM_STRING(filename)), 0);
     } SCM_WHEN_ERROR {
         (void)SCM_INTERNAL_MUTEX_LOCK(ldinfo.prov_mutex);
         ldinfo.providing = Scm_AssocDeleteX(feature, ldinfo.providing, SCM_CMP_EQUAL);
@@ -842,14 +896,15 @@ static void autoload_print(ScmObj obj, ScmPort *out, ScmWriteContext *ctx)
 
 SCM_DEFINE_BUILTIN_CLASS_SIMPLE(Scm_AutoloadClass, autoload_print);
 
-ScmObj Scm_MakeAutoload(ScmSymbol *name,
+ScmObj Scm_MakeAutoload(ScmModule *where,
+                        ScmSymbol *name,
                         ScmString *path,
                         ScmSymbol *import_from)
 {
     ScmAutoload *adata = SCM_NEW(ScmAutoload);
     SCM_SET_CLASS(adata, SCM_CLASS_AUTOLOAD);
     adata->name = name;
-    adata->module = SCM_CURRENT_MODULE();
+    adata->module = where;
     adata->path = path;
     adata->import_from = import_from;
     adata->loaded = FALSE;
@@ -881,13 +936,14 @@ void Scm_DefineAutoload(ScmModule *where,
         ScmObj entry = SCM_CAR(ep);
         if (SCM_SYMBOLP(entry)) {
             Scm_Define(where, SCM_SYMBOL(entry),
-                       Scm_MakeAutoload(SCM_SYMBOL(entry), path, import_from));
+                       Scm_MakeAutoload(where, SCM_SYMBOL(entry),
+                                        path, import_from));
         } else if (SCM_PAIRP(entry)
                    && SCM_EQ(key_macro, SCM_CAR(entry))
                    && SCM_PAIRP(SCM_CDR(entry))
                    && SCM_SYMBOLP(SCM_CADR(entry))) {
             ScmSymbol *sym = SCM_SYMBOL(SCM_CADR(entry));
-            ScmObj autoload = Scm_MakeAutoload(sym, path, import_from);
+            ScmObj autoload = Scm_MakeAutoload(where, sym, path, import_from);
             Scm_Define(where, sym,
                        Scm_MakeMacroAutoload(sym, SCM_AUTOLOAD(autoload)));
         } else {
@@ -989,21 +1045,37 @@ ScmObj Scm_LoadAutoload(ScmAutoload *adata)
 void Scm__InitLoad(void)
 {
     ScmModule *m = Scm_SchemeModule();
-    ScmObj init_load_path, init_dynload_path, init_load_suffixes, t;
+    ScmObj init_load_path, init_dynload_path, init_load_suffixes,
+        init_cond_features, t;
 
     init_load_path = t = SCM_NIL;
     SCM_APPEND(init_load_path, t, break_env_paths("GAUCHE_LOAD_PATH"));
-    SCM_APPEND1(init_load_path, t, SCM_MAKE_STR(GAUCHE_SITE_LIB_DIR));
-    SCM_APPEND1(init_load_path, t, SCM_MAKE_STR(GAUCHE_LIB_DIR));
+    SCM_APPEND1(init_load_path, t, Scm_SiteLibraryDirectory());
+    SCM_APPEND1(init_load_path, t, Scm_LibraryDirectory());
 
     init_dynload_path = t = SCM_NIL;
     SCM_APPEND(init_dynload_path, t, break_env_paths("GAUCHE_DYNLOAD_PATH"));
-    SCM_APPEND1(init_dynload_path, t, SCM_MAKE_STR(GAUCHE_SITE_ARCH_DIR));
-    SCM_APPEND1(init_dynload_path, t, SCM_MAKE_STR(GAUCHE_ARCH_DIR));
+    SCM_APPEND1(init_dynload_path, t, Scm_SiteArchitectureDirectory());
+    SCM_APPEND1(init_dynload_path, t, Scm_ArchitectureDirectory());
 
     init_load_suffixes = t = SCM_NIL;
     SCM_APPEND1(init_load_suffixes, t, SCM_MAKE_STR(LOAD_SUFFIX));
 
+    init_cond_features = t = SCM_NIL;
+    SCM_APPEND1(init_cond_features, t, SCM_LIST1(SCM_SYM_GAUCHE));
+#ifdef __MINGW32__
+    SCM_APPEND1(init_cond_features, t, SCM_LIST1(SCM_SYM_GAUCHE_WINDOWS));
+#endif /*__MINGW32__*/
+#if defined(GAUCHE_CHAR_ENCODING_EUC_JP)
+    SCM_APPEND1(init_cond_features, t, SCM_LIST1(SCM_SYM_GAUCHE_EUCJP));
+#elif defined(GAUCHE_CHAR_ENCODING_SJIS)
+    SCM_APPEND1(init_cond_features, t, SCM_LIST1(SCM_SYM_GAUCHE_SJIS));
+#elif defined(GAUCHE_CHAR_ENCODING_UTF8)
+    SCM_APPEND1(init_cond_features, t, SCM_LIST1(SCM_SYM_GAUCHE_UTF8));
+#else
+    SCM_APPEND1(init_cond_features, t, SCM_LIST1(SCM_SYM_GAUCHE_NONE));
+#endif
+
     (void)SCM_INTERNAL_MUTEX_INIT(ldinfo.path_mutex);
     (void)SCM_INTERNAL_MUTEX_INIT(ldinfo.prov_mutex);
     (void)SCM_INTERNAL_COND_INIT(ldinfo.prov_cv);
@@ -1013,6 +1085,7 @@ void Scm__InitLoad(void)
     key_error_if_not_found = SCM_MAKE_KEYWORD("error-if-not-found");
     key_environment = SCM_MAKE_KEYWORD("environment");
     key_macro = SCM_MAKE_KEYWORD("macro");
+    key_ignore_coding = SCM_MAKE_KEYWORD("ignore-coding");
     
     SCM_DEFINE(m, "load-from-port", SCM_OBJ(&load_from_port_STUB));
     SCM_DEFINE(m, "load", SCM_OBJ(&load_STUB));
@@ -1023,6 +1096,7 @@ void Scm__InitLoad(void)
     DEF(ldinfo.load_path_rec,    SCM_SYM_LOAD_PATH, init_load_path);
     DEF(ldinfo.dynload_path_rec, SCM_SYM_DYNAMIC_LOAD_PATH, init_dynload_path);
     DEF(ldinfo.load_suffixes_rec, SCM_SYM_LOAD_SUFFIXES, init_load_suffixes);
+    DEF(ldinfo.cond_features_rec, SCM_SYM_COND_FEATURES, init_cond_features);
 
     ldinfo.provided = SCM_LIST4(SCM_MAKE_STR("srfi-6"), /* string ports (builtin) */
                                 SCM_MAKE_STR("srfi-8"), /* receive (builtin) */
