@@ -30,7 +30,7 @@
  *   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  *   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- *  $Id: macro.c,v 1.47 2004/01/18 12:07:31 shirok Exp $
+ *  $Id: macro.c,v 1.51 2004/08/16 02:33:58 shirok Exp $
  */
 
 #define LIBGAUCHE_BODY
@@ -59,6 +59,28 @@ ScmObj Scm_MakeSyntax(ScmSymbol *name, ScmCompileProc compiler, void *data)
     return SCM_OBJ(s);
 }
 
+/*===================================================================
+ * Macro object
+ */
+
+static void macro_print(ScmObj obj, ScmPort *port, ScmWriteContext *mode)
+{
+    Scm_Printf(port, "#<macro %A>", SCM_MACRO(obj)->name);
+}
+
+SCM_DEFINE_BUILTIN_CLASS_SIMPLE(Scm_MacroClass, macro_print);
+
+ScmObj Scm_MakeMacro(ScmSymbol *name, ScmTransformerProc transformer,
+                     void *data)
+{
+    ScmMacro *s = SCM_NEW(ScmMacro);
+    SCM_SET_CLASS(s, SCM_CLASS_MACRO);
+    s->name = name;
+    s->transformer = transformer;
+    s->data = data;
+    return SCM_OBJ(s);
+}
+
 /*===================================================================
  * SyntaxPattern object
  *   Internal object to construct pattern matcher
@@ -125,38 +147,40 @@ ScmSyntaxRules *make_syntax_rules(int nr)
 /* TODO: better error message on syntax error (macro invocation with
    bad number of arguments) */
 
-static ScmObj macro_transform(ScmObj form, ScmObj env, int ctx,
-                              int *depth, void *data)
+static ScmObj macro_transform(ScmObj self, ScmObj form, ScmObj env, void *data)
 {
     ScmObj proc = SCM_OBJ(data);
     SCM_ASSERT(SCM_PAIRP(form));
-    return Scm_Compile(Scm_Apply(proc, SCM_CDR(form)), env, ctx);
+    return Scm_Apply(proc, SCM_CDR(form));
 }
 
 ScmObj Scm_MakeMacroTransformer(ScmSymbol *name, ScmProcedure *proc)
 {
-    return Scm_MakeSyntax(name, macro_transform, (void*)proc);
+    return Scm_MakeMacro(name, macro_transform, (void*)proc);
 }
 
-static ScmObj macro_autoload(ScmObj form, ScmObj env, int ctx,
-                             int *depth, void *data)
+static ScmMacro *resolve_macro_autoload(ScmAutoload *adata)
 {
-    ScmAutoload *adata = SCM_AUTOLOAD(data);
-    ScmObj syn = Scm_LoadAutoload(adata);
-    if (!SCM_SYNTAXP(syn)) {
-        Scm_Error("tried to autoload macro %S, but it yields non-macro object: %S", adata->name, syn);
+    ScmObj mac = Scm_LoadAutoload(adata);
+    if (!SCM_MACROP(mac)) {
+        Scm_Error("tried to autoload macro %S, but it yields non-macro object: %S", adata->name, mac);
     }
-    return SCM_SYNTAX(syn)->compiler(form, env, ctx, depth,
-                                     SCM_SYNTAX(syn)->data);
+    return SCM_MACRO(mac);
+}
+
+static ScmObj macro_autoload(ScmObj self, ScmObj form, ScmObj env, void *data)
+{
+    ScmMacro *mac = resolve_macro_autoload(SCM_AUTOLOAD(data));
+    return mac->transformer(SCM_OBJ(mac), form, env, mac->data);
 }
 
 ScmObj Scm_MakeMacroAutoload(ScmSymbol *name, ScmAutoload *adata)
 {
-    return Scm_MakeSyntax(name, macro_autoload, (void*)adata);
+    return Scm_MakeMacro(name, macro_autoload, (void*)adata);
 }
 
 static ScmObj compile_define_macro(ScmObj form, ScmObj env, int ctx,
-                                   int *depth, void *data)
+                                   void *data)
 {
     ScmObj name, trans, mt = SCM_NIL;
     int len;
@@ -865,12 +889,11 @@ static ScmObj synrule_expand(ScmObj form, ScmObj env, ScmSyntaxRules *sr)
     return SCM_NIL;
 }
 
-static ScmObj synrule_transform(ScmObj form, ScmObj env,
-                                int ctx, int *depth, void *data)
+static ScmObj synrule_transform(ScmObj self, ScmObj form, ScmObj env,
+                                void *data)
 {
     ScmSyntaxRules *sr = (ScmSyntaxRules *)data;
-    ScmObj expanded = synrule_expand(form, env, sr);
-    return Scm_Compile(expanded, env, ctx /*,depth*/);
+    return synrule_expand(form, env, sr);
 }
 
 /*-------------------------------------------------------------------
@@ -878,7 +901,7 @@ static ScmObj synrule_transform(ScmObj form, ScmObj env,
  *    Internal macro of syntax-rules.  Taking macro name as the first arg.
  */
 static ScmObj compile_syntax_rules(ScmObj form, ScmObj env,
-                                   int ctx, int *depth, void *data)
+                                   int ctx, void *data)
 {
     ScmObj name, literals, rules;
     ScmSyntaxRules *sr;
@@ -897,9 +920,9 @@ static ScmObj compile_syntax_rules(ScmObj form, ScmObj env,
 #ifdef DEBUG_SYNRULE
     Scm_Printf(SCM_CUROUT, "%S\n", sr);
 #endif
-    return SCM_LIST1(Scm_MakeSyntax(SCM_SYMBOL(name),
-                                    synrule_transform,
-                                    (void*)sr));
+    return SCM_LIST1(Scm_MakeMacro(SCM_SYMBOL(name),
+                                   synrule_transform,
+                                   (void*)sr));
         
   badform:
     Scm_Error("malformed syntax-rules: ",
@@ -919,7 +942,7 @@ static ScmSyntax syntax_syntax_rules = {
  */
 
 static ScmObj compile_define_syntax(ScmObj form, ScmObj env, int ctx,
-                                    int *depth, void *data)
+                                    void *data)
 {
     ScmObj var, body, synrule;
     if (Scm_Length(form) != 3) Scm_Error("malformed define-syntax: %S", form);
@@ -935,7 +958,7 @@ static ScmObj compile_define_syntax(ScmObj form, ScmObj env, int ctx,
         Scm_Error("define-syntax needs a syntax-rules form, but got %S", body);
     synrule = compile_syntax_rules(Scm_Cons(SCM_SYM_SYNTAX_RULES_INT,
                                             Scm_Cons(var, SCM_CDR(body))),
-                                   env, ctx, depth, NULL);
+                                   env, ctx, NULL);
     SCM_ASSERT(SCM_PAIRP(synrule));
     Scm_Define(SCM_CURRENT_MODULE(), SCM_SYMBOL(var), SCM_CAR(synrule));
     return synrule;
@@ -954,7 +977,7 @@ static ScmSyntax syntax_define_syntax = {
  */
 
 static ScmObj compile_let_syntax(ScmObj form, ScmObj env, int ctx,
-                                 int *depth, void *data)
+                                 void *data)
 {
     int letrecp = (data != NULL);
     ScmObj var, rule, vars = SCM_NIL, vars_t = SCM_NIL;
@@ -1000,7 +1023,7 @@ static ScmObj compile_let_syntax(ScmObj form, ScmObj env, int ctx,
         rule = SCM_CDAR(sp);
         synrule = compile_syntax_rules(Scm_Cons(SCM_SYM_SYNTAX_RULES_INT,
                                                 Scm_Cons(var, SCM_CDR(rule))),
-                                       newenv, ctx, depth, NULL);
+                                       newenv, ctx, NULL);
         SCM_ASSERT(SCM_PAIRP(synrule));
         SCM_SET_CDR(SCM_CAR(sp), SCM_CAR(synrule));
     }
@@ -1031,18 +1054,18 @@ static ScmSyntax syntax_letrec_syntax = {
 ScmObj Scm_MacroExpand(ScmObj expr, ScmObj env, int oncep)
 {
     ScmObj sym;
-    ScmSyntax *syn;
+    ScmMacro *mac;
 
     for (;;) {
         if (!SCM_PAIRP(expr)) return expr;
         if (!SCM_SYMBOLP(SCM_CAR(expr)) && !SCM_IDENTIFIERP(SCM_CAR(expr)))
             return expr;
 
-        syn = NULL;
+        mac = NULL;
         sym = Scm_CompileLookupEnv(SCM_CAR(expr), env, TRUE);
-        if (SCM_SYNTAXP(sym)) {
+        if (SCM_MACROP(sym)) {
             /* local syntactic binding */
-            syn = SCM_SYNTAX(sym);
+            mac = SCM_MACRO(sym);
         } else {
             if (SCM_IDENTIFIERP(sym)) {
                 sym = SCM_OBJ(SCM_IDENTIFIER(sym)->name);
@@ -1051,21 +1074,13 @@ ScmObj Scm_MacroExpand(ScmObj expr, ScmObj env, int oncep)
                 ScmGloc *g = Scm_FindBinding(Scm_VM()->module, SCM_SYMBOL(sym), FALSE);
                 if (g) {
                     ScmObj gv = SCM_GLOC_GET(g);
-                    if (SCM_SYNTAXP(gv)) syn = SCM_SYNTAX(gv);
+                    if (SCM_MACROP(gv)) mac = SCM_MACRO(gv);
                 }
             }
         }
-        if (syn) {
-            if (syn->compiler == macro_transform) {
-                ScmObj proc = SCM_OBJ(syn->data);
-                expr = Scm_Apply(proc, SCM_CDR(expr));
-                if (!oncep) continue;
-            }
-            if (syn->compiler == synrule_transform) {
-                ScmSyntaxRules *sr = (ScmSyntaxRules *)syn->data;
-                expr = synrule_expand(expr, env, sr);
-                if (!oncep) continue;
-            }
+        if (mac) {
+            expr = mac->transformer(SCM_OBJ(mac), expr, env, mac->data);
+            if (!oncep) continue;
         }
         break;
     }
@@ -1077,7 +1092,7 @@ ScmObj Scm_MacroExpand(ScmObj expr, ScmObj env, int oncep)
  * From scheme, this syntax is visible as %macro-expand.
  */
 static ScmObj compile_macro_expand(ScmObj form, ScmObj env,
-                                   int ctx, int *depth, void *data)
+                                   int ctx, void *data)
 {
     int oncep = (data != NULL);
     if (!SCM_PAIRP(SCM_CDR(form)) || !SCM_NULLP(SCM_CDDR(form)))
