@@ -1,7 +1,7 @@
 /*
  * main.c - interpreter main program
  *
- *   Copyright (c) 2000-2003 Shiro Kawai, All rights reserved.
+ *   Copyright (c) 2000-2004 Shiro Kawai, All rights reserved.
  * 
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -30,7 +30,7 @@
  *   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  *   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- *  $Id: main.c,v 1.74 2004/01/02 13:15:57 shirok Exp $
+ *  $Id: main.c,v 1.76 2004/09/17 23:32:16 shirok Exp $
  */
 
 #include <unistd.h>
@@ -162,9 +162,15 @@ static void sig_setup(void)
     sigfillset(&set);
     sigdelset(&set, SIGABRT);
     sigdelset(&set, SIGILL);
+#ifdef SIGKILL
     sigdelset(&set, SIGKILL);
+#endif
+#ifdef SIGCONT
     sigdelset(&set, SIGCONT);
+#endif
+#ifdef SIGSTOP
     sigdelset(&set, SIGSTOP);
+#endif
     sigdelset(&set, SIGSEGV);
 #ifdef SIGPROF
     sigdelset(&set, SIGPROF);
@@ -196,7 +202,7 @@ int main(int argc, char **argv)
     const char *scriptfile = NULL;
     ScmObj av = SCM_NIL;
 
-#ifdef __CYGWIN__
+#if defined(__CYGWIN__) || defined(__MINGW32__)
     /* Cygwin needs explicit initialization for GC module.
        This code is taken from gc.h and gcconfig.h (I don't want to
        include private/gcconfig.h)
@@ -236,7 +242,7 @@ int main(int argc, char **argv)
     /* load init file */
     if (load_initfile) {
         SCM_UNWIND_PROTECT {
-            Scm_Load("gauche-init.scm", TRUE);
+            Scm_Load("gauche-init.scm", 0);
         }
         SCM_WHEN_ERROR {
             fprintf(stderr, "Error in initialization file.\n");
@@ -296,7 +302,7 @@ int main(int argc, char **argv)
             Scm_AddLoadPath(Scm_GetStringConst(SCM_STRING(v)), TRUE);
             break;
         case 'l':
-            Scm_Load(Scm_GetStringConst(SCM_STRING(v)), TRUE);
+            Scm_Load(Scm_GetStringConst(SCM_STRING(v)), 0);
             break;
         case 'u':
             Scm_Require(Scm_StringJoin(Scm_StringSplitByChar(SCM_STRING(v),
@@ -324,7 +330,7 @@ int main(int argc, char **argv)
     if (scriptfile != NULL) {
         ScmObj result, mainproc;
         
-        Scm_Load(scriptfile, TRUE);
+        Scm_Load(scriptfile, 0);
 
         /* if symbol 'main is bound to a procedure in the user module,
            call it.  (SRFI-22) */
@@ -352,7 +358,7 @@ int main(int argc, char **argv)
     }
 
     if (batch_mode || (!isatty(0) && !interactive_mode)) {
-        Scm_LoadFromPort(SCM_PORT(Scm_Stdin()));
+        Scm_LoadFromPort(SCM_PORT(Scm_Stdin()), 0);
     } else {
         Scm_Repl(SCM_FALSE, SCM_FALSE, SCM_FALSE, SCM_FALSE);
     }
