@@ -30,7 +30,7 @@
  *   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  *   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- *  $Id: module.c,v 1.47 2004/01/18 12:07:31 shirok Exp $
+ *  $Id: module.c,v 1.51 2004/07/26 20:16:08 shirok Exp $
  */
 
 #define LIBGAUCHE_BODY
@@ -56,9 +56,21 @@
  *  it, recovering its resouces.
  */
 
-static ScmObj anon_module_name = SCM_UNBOUND; /* Name used for anonymous
-                                                 modules.  Symbol '#',
-                                                 set by init */
+/* Mutex of module operation
+ *
+ * [SK] Each module used to have a mutex for accesses to it.  I changed it
+ * to use a single global lock (modules.mutex), based on the following
+ * observations:
+ *
+ *  - Profiling shows mutex_lock takes around 10% of program loading
+ *    phase.
+ *
+ *  - Module operations almost always occur during program loading and
+ *    interactive session.  Having giant lock for module operations won't
+ *    affect normal runtime performance.
+ *
+ * Benchmark showed the change made program loading 30% faster.
+ */
 
 static void module_print(ScmObj obj, ScmPort *port, ScmWriteContext *ctx)
 {
@@ -71,10 +83,12 @@ SCM_DEFINE_BUILTIN_CLASS(Scm_ModuleClass,
 
 /* Global module table */
 static struct {
+    ScmObj anon_name;       /* Name used for anonymous modules.
+                               Symbol '#', set by init */
     ScmHashTable *table;    /* Maps name -> module. */
     ScmInternalMutex mutex; /* Lock for table.  Only register_module and
                                lookup_module may hold the lock. */
-} modules = { NULL };
+} modules = { SCM_UNBOUND, NULL };
 
 /* Predefined modules - slots will be initialized by Scm__InitModule */
 #define DEFINE_STATIC_MODULE(cname) \
@@ -100,7 +114,6 @@ static void init_module(ScmModule *m, ScmSymbol *name)
     m->parents = defaultParents;
     m->mpl = Scm_Cons(SCM_OBJ(m), defaultMpl);
     m->table = SCM_HASHTABLE(Scm_MakeHashTable(SCM_HASH_ADDRESS, NULL, 0));
-    (void)SCM_INTERNAL_MUTEX_INIT(m->mutex);
 }
 
 /* Internal */
@@ -143,8 +156,8 @@ static ScmModule *lookup_module_create(ScmSymbol *name, int *created)
 ScmObj Scm_MakeModule(ScmSymbol *name, int error_if_exists)
 {
     ScmObj r;
-    if (name == NULL) name = SCM_SYMBOL(anon_module_name);
-    if (SCM_EQ(SCM_OBJ(name), anon_module_name)) {
+    if (name == NULL) name = SCM_SYMBOL(modules.anon_name);
+    if (SCM_EQ(SCM_OBJ(name), modules.anon_name)) {
         r = make_module(name);
     } else {
         int created;
@@ -165,18 +178,27 @@ ScmObj Scm_MakeModule(ScmSymbol *name, int error_if_exists)
  * Finding and modifying bindings
  */
 
+#define SEARCHED_ARRAY_SIZE  64
+
 ScmGloc *Scm_FindBinding(ScmModule *module, ScmSymbol *symbol,
                          int stay_in_module)
 {
     ScmHashEntry *e;
     ScmModule *m = module;
-    ScmObj p, mp, searched = SCM_NIL;
+    ScmObj p, mp;
+    ScmGloc *gloc = NULL;
+
+    /* keep record of searched modules.  we use stack array for small # of
+       modules, in order to avoid consing for typical cases. */
+    ScmObj searched[SEARCHED_ARRAY_SIZE];
+    int num_searched = 0, i;
+    ScmObj more_searched = SCM_NIL;
+
+    (void)SCM_INTERNAL_MUTEX_LOCK(modules.mutex);
 
     /* fist, search from the specified module */
-    (void)SCM_INTERNAL_MUTEX_LOCK(m->mutex);
     e = Scm_HashTableGet(m->table, SCM_OBJ(symbol));
-    (void)SCM_INTERNAL_MUTEX_UNLOCK(m->mutex);
-    if (e) return SCM_GLOC(e->value);
+    if (e) { gloc = SCM_GLOC(e->value); goto found; }
     
     if (!stay_in_module) {
         /* Next, search from imported modules */
@@ -184,32 +206,45 @@ ScmGloc *Scm_FindBinding(ScmModule *module, ScmSymbol *symbol,
             SCM_ASSERT(SCM_MODULEP(SCM_CAR(p)));
             SCM_FOR_EACH(mp, SCM_MODULE(SCM_CAR(p))->mpl) {
                 SCM_ASSERT(SCM_MODULEP(SCM_CAR(mp)));
-                if (!SCM_FALSEP(Scm_Memq(SCM_CAR(mp), searched))) break;
+                
+                for (i=0; i<num_searched; i++) {
+                    if (SCM_EQ(SCM_CAR(mp), searched[i])) goto skip;
+                }
+                if (!SCM_NULLP(more_searched)) {
+                    if (!SCM_FALSEP(Scm_Memq(SCM_CAR(mp), more_searched))) {
+                        goto skip;
+                    }
+                }
                 
                 m = SCM_MODULE(SCM_CAR(mp));
-                (void)SCM_INTERNAL_MUTEX_LOCK(m->mutex);
                 e = Scm_HashTableGet(m->table, SCM_OBJ(symbol));
-                (void)SCM_INTERNAL_MUTEX_UNLOCK(m->mutex);
                 if (e &&
                     (SCM_TRUEP(m->exported)
                      || !SCM_FALSEP(Scm_Memq(SCM_OBJ(symbol), m->exported)))) {
-                    return SCM_GLOC(e->value);
+                    gloc = SCM_GLOC(e->value);
+                    goto found;
+                }
+
+                if (num_searched < SEARCHED_ARRAY_SIZE) {
+                    searched[num_searched++] = SCM_OBJ(m);
+                } else {
+                    more_searched = Scm_Cons(SCM_OBJ(m), more_searched);
                 }
-                searched = Scm_Cons(SCM_OBJ(m), searched);
             }
+          skip:;
         }
         /* Then, search from parent modules */
         SCM_ASSERT(SCM_PAIRP(module->mpl));
         SCM_FOR_EACH(mp, SCM_CDR(module->mpl)) {
             SCM_ASSERT(SCM_MODULEP(SCM_CAR(mp)));
             m = SCM_MODULE(SCM_CAR(mp));
-            (void)SCM_INTERNAL_MUTEX_LOCK(m->mutex);
             e = Scm_HashTableGet(m->table, SCM_OBJ(symbol));
-            (void)SCM_INTERNAL_MUTEX_UNLOCK(m->mutex);
-            if (e) return SCM_GLOC(e->value);
+            if (e) { gloc = SCM_GLOC(e->value); goto found; }
         }
     }
-    return NULL;
+  found:
+    (void)SCM_INTERNAL_MUTEX_UNLOCK(modules.mutex);
+    return gloc;
 }
 
 ScmObj Scm_SymbolValue(ScmModule *module, ScmSymbol *symbol)
@@ -228,7 +263,7 @@ ScmObj Scm_Define(ScmModule *module, ScmSymbol *symbol, ScmObj value)
     ScmHashEntry *e;
     int redefining = FALSE;
     
-    (void)SCM_INTERNAL_MUTEX_LOCK(module->mutex);
+    (void)SCM_INTERNAL_MUTEX_LOCK(modules.mutex);
     e = Scm_HashTableGet(module->table, SCM_OBJ(symbol));
     if (e) {
         g = SCM_GLOC(e->value);
@@ -242,7 +277,7 @@ ScmObj Scm_Define(ScmModule *module, ScmSymbol *symbol, ScmObj value)
         SCM_GLOC_SET(g, value);
         Scm_HashTablePut(module->table, SCM_OBJ(symbol), SCM_OBJ(g));
     }
-    (void)SCM_INTERNAL_MUTEX_UNLOCK(module->mutex);
+    (void)SCM_INTERNAL_MUTEX_UNLOCK(modules.mutex);
     
     if (redefining) {
         Scm_Warn("redefining constant %S::%S", g->module, g->name);
@@ -257,7 +292,7 @@ ScmObj Scm_DefineConst(ScmModule *module, ScmSymbol *symbol, ScmObj value)
     ScmObj oldval = SCM_UNDEFINED;
     int redefining = FALSE;
 
-    (void)SCM_INTERNAL_MUTEX_LOCK(module->mutex);
+    (void)SCM_INTERNAL_MUTEX_LOCK(modules.mutex);
     e = Scm_HashTableGet(module->table, SCM_OBJ(symbol));
     /* NB: this function bypasses check of gloc setter */
     if (e) {
@@ -273,7 +308,7 @@ ScmObj Scm_DefineConst(ScmModule *module, ScmSymbol *symbol, ScmObj value)
         g->value = value;
         Scm_HashTablePut(module->table, SCM_OBJ(symbol), SCM_OBJ(g));
     }
-    (void)SCM_INTERNAL_MUTEX_UNLOCK(module->mutex);
+    (void)SCM_INTERNAL_MUTEX_UNLOCK(modules.mutex);
 
     if (redefining && !Scm_EqualP(value, oldval)) {
         Scm_Warn("redefining constant %S::%S", g->module->name, g->name);
@@ -295,10 +330,10 @@ ScmObj Scm_ImportModules(ScmModule *module, ScmObj list)
         }
         mod = Scm_FindModule(name, FALSE);
         if (!SCM_MODULEP(mod)) Scm_Error("no such module: %S", SCM_CAR(lp));
-        (void)SCM_INTERNAL_MUTEX_LOCK(module->mutex);
+        (void)SCM_INTERNAL_MUTEX_LOCK(modules.mutex);
         module->imported =
             Scm_Cons(mod, Scm_DeleteX(mod, module->imported, SCM_CMP_EQ));
-        (void)SCM_INTERNAL_MUTEX_UNLOCK(module->mutex);
+        (void)SCM_INTERNAL_MUTEX_UNLOCK(modules.mutex);
     }
     return module->imported;
 }
@@ -308,7 +343,7 @@ ScmObj Scm_ExportSymbols(ScmModule *module, ScmObj list)
     ScmObj lp, syms, badsym = SCM_FALSE;
     int error = FALSE;
 
-    (void)SCM_INTERNAL_MUTEX_LOCK(module->mutex);
+    (void)SCM_INTERNAL_MUTEX_LOCK(modules.mutex);
     syms = module->exported;
     if (!SCM_TRUEP(syms)) {
         SCM_FOR_EACH(lp, list) {
@@ -322,16 +357,16 @@ ScmObj Scm_ExportSymbols(ScmModule *module, ScmObj list)
         }
         if (!error) module->exported = syms;
     }
-    (void)SCM_INTERNAL_MUTEX_UNLOCK(module->mutex);
+    (void)SCM_INTERNAL_MUTEX_UNLOCK(modules.mutex);
     if (error) Scm_Error("symbol required, but got %S", badsym);
     return syms;
 }
 
 ScmObj Scm_ExportAll(ScmModule *module)
 {
-    (void)SCM_INTERNAL_MUTEX_LOCK(module->mutex);
+    (void)SCM_INTERNAL_MUTEX_LOCK(modules.mutex);
     module->exported = SCM_TRUE;
-    (void)SCM_INTERNAL_MUTEX_UNLOCK(module->mutex);
+    (void)SCM_INTERNAL_MUTEX_UNLOCK(modules.mutex);
     return SCM_OBJ(module);
 }
 
@@ -502,5 +537,5 @@ void Scm__InitModule(void)
     defaultParents = SCM_LIST1(SCM_CAR(mpl));
     defaultMpl = mpl;
 
-    anon_module_name = SCM_SYM_SHARP;
+    modules.anon_name = SCM_SYM_SHARP;
 }
