@@ -30,7 +30,7 @@
  *   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  *   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- *  $Id: port.c,v 1.99 2004/02/17 13:22:58 shirok Exp $
+ *  $Id: port.c,v 1.118 2004/12/02 12:59:28 shirok Exp $
  */
 
 #include <unistd.h>
@@ -39,6 +39,7 @@
 #include <errno.h>
 #define LIBGAUCHE_BODY
 #include "gauche.h"
+#include "gauche/class.h"
 #include "gauche/port.h"
 
 #undef MAX
@@ -55,12 +56,22 @@ static void port_finalize(ScmObj obj, void* data);
 static void register_buffered_port(ScmPort *port);
 static void unregister_buffered_port(ScmPort *port);
 static void bufport_flush(ScmPort*, int, int);
-static int file_closer(ScmPort *p);
+static void file_closer(ScmPort *p);
 
 SCM_DEFINE_BASE_CLASS(Scm_PortClass,
                       ScmPort, /* instance type */
                       port_print, NULL, NULL, NULL, NULL);
 
+static ScmClass *port_cpl[] = {
+    SCM_CLASS_STATIC_PTR(Scm_PortClass),
+    SCM_CLASS_STATIC_PTR(Scm_TopClass),
+    NULL
+};
+
+SCM_DEFINE_BASE_CLASS(Scm_CodingAwarePortClass,
+                      ScmPort, /* instance type */
+                      port_print, NULL, NULL, NULL, port_cpl);
+
 /*================================================================
  * Common
  */
@@ -73,12 +84,13 @@ SCM_DEFINE_BASE_CLASS(Scm_PortClass,
  *   may share the same file descriptor for efficiency (e.g. stdios).
  *   In such cases, it is C code's responsibility to destroy the port.
  */
-static int port_cleanup(ScmPort *port)
+static void port_cleanup(ScmPort *port)
 {
-    if (SCM_PORT_CLOSED_P(port)) return 0;
+    if (SCM_PORT_CLOSED_P(port)) return;
     switch (SCM_PORT_TYPE(port)) {
     case SCM_PORT_FILE:
-        if (SCM_PORT_DIR(port) == SCM_PORT_OUTPUT && !SCM_PORT_ERROR_P(port)) {
+        if (SCM_PORT_DIR(port) == SCM_PORT_OUTPUT
+            && !SCM_PORT_ERROR_OCCURRED_P(port)) {
             bufport_flush(port, 0, TRUE);
         }
         if (port->ownerp && port->src.buf.closer) port->src.buf.closer(port);
@@ -90,7 +102,8 @@ static int port_cleanup(ScmPort *port)
         break;
     }
     SCM_PORT_CLOSED_P(port) = TRUE;
-    return 0;
+    /* avoid unnecessary finalization */
+    Scm_UnregisterFinalizer(SCM_OBJ(port));
 }
 
 /* called by GC */
@@ -104,12 +117,12 @@ static void port_finalize(ScmObj obj, void* data)
  *   If this port owns the underlying file descriptor/stream, 
  *   ownerp must be TRUE.
  */
-static ScmPort *make_port(int dir, int type)
+static ScmPort *make_port(ScmClass *klass, int dir, int type)
 {
     ScmPort *port;
 
-    port = SCM_NEW(ScmPort);
-    SCM_SET_CLASS(port, SCM_CLASS_PORT);
+    port = SCM_ALLOCATE(ScmPort, klass);
+    SCM_SET_CLASS(port, klass);
     port->direction = dir;
     port->type = type;
     port->scrcnt = 0;
@@ -124,6 +137,7 @@ static ScmPort *make_port(int dir, int type)
     port->lockOwner = NULL;
     port->lockCount = 0;
     port->data = SCM_FALSE;
+    port->line = 1;
     switch (type) {
     case SCM_PORT_FILE: /*FALLTHROUGH*/;
     case SCM_PORT_PROC:
@@ -138,15 +152,14 @@ static ScmPort *make_port(int dir, int type)
 /*
  * Close
  */
-ScmObj Scm_ClosePort(ScmPort *port)
+void Scm_ClosePort(ScmPort *port)
 {
-    int result = 0;
     ScmVM *vm = Scm_VM();
     PORT_LOCK(port, vm);
     PORT_SAFE_CALL(port,
                    do {
                        if (!SCM_PORT_CLOSED_P(port)) {
-                           result = port_cleanup(port);
+                           port_cleanup(port);
                            if (SCM_PORT_TYPE(port) == SCM_PORT_FILE
                                && SCM_PORT_DIR(port) == SCM_PORT_OUTPUT) {
                                unregister_buffered_port(port);
@@ -154,7 +167,6 @@ ScmObj Scm_ClosePort(ScmPort *port)
                        }
                    } while (0));
     PORT_UNLOCK(port);
-    return result? SCM_FALSE : SCM_TRUE;
 }
 
 /*
@@ -194,19 +206,7 @@ ScmObj Scm_PortName(ScmPort *port)
 
 int Scm_PortLine(ScmPort *port)
 {
-    switch (SCM_PORT_TYPE(port)) {
-    case SCM_PORT_FILE:
-        return port->src.buf.line;
-    default:
-        /* TODO: proc port to customize */
-        return -1;
-    }
-}
-
-int Scm_PortPosition(ScmPort *port)
-{
-    /* TODO: WRITEME */
-    return -1;
+    return port->line;
 }
 
 static void port_print(ScmObj obj, ScmPort *port, ScmWriteContext *ctx)
@@ -382,6 +382,8 @@ int Scm_FdReady(int fd, int dir)
  *    If not, it calls port->src.buf.ready if it is not NULL to query
  *    the character is ready.   If port->src.buf.ready is NULL, bufport
  *    assumes the input is always ready.
+ *    port->src.buf.ready should return either SCM_FD_READY, SCM_FD_WOULDBLOCK
+ *    or SCM_FD_UNKNOWN.
  *
  *  Filenum
  *    Port->src.buf.filenum is a query procedure that should return the
@@ -429,7 +431,8 @@ int Scm_FdReady(int fd, int dir)
 
 #define SCM_PORT_DEFAULT_BUFSIZ 8192
 
-ScmObj Scm_MakeBufferedPort(ScmObj name,
+ScmObj Scm_MakeBufferedPort(ScmClass *klass,
+                            ScmObj name,
                             int dir,     /* direction */
                             int ownerp,  /* owner flag*/
                             ScmPortBuffer *bufrec)
@@ -440,7 +443,7 @@ ScmObj Scm_MakeBufferedPort(ScmObj name,
     
     if (size <= 0) size = SCM_PORT_DEFAULT_BUFSIZ;
     if (buf == NULL) buf = SCM_NEW_ATOMIC2(char*, size);
-    p = make_port(dir, SCM_PORT_FILE);
+    p = make_port(klass, dir, SCM_PORT_FILE);
     p->name = name;
     p->ownerp = ownerp;
     p->src.buf.buffer = buf;
@@ -460,7 +463,6 @@ ScmObj Scm_MakeBufferedPort(ScmObj name,
     p->src.buf.filenum = bufrec->filenum;
     p->src.buf.seeker = bufrec->seeker;
     p->src.buf.data = bufrec->data;
-    p->src.buf.line = 1;
     if (dir == SCM_PORT_OUTPUT) register_buffered_port(p);
     return SCM_OBJ(p);
 }
@@ -482,7 +484,8 @@ static void bufport_flush(ScmPort *p, int cnt, int forcep)
         p->error = TRUE;
         /* TODO: can we raise an error here, or should we propagate
            it to the caller? */
-        Scm_Error("Couldn't flush port %S due to an error", p);
+        Scm_PortError(p, SCM_PORT_ERROR_OUTPUT,
+                      "Couldn't flush port %S due to an error", p);
     }
     if (nwrote >= 0 && nwrote < cursiz) {
         memmove(p->src.buf.buffer, p->src.buf.buffer+nwrote,
@@ -530,7 +533,7 @@ static int bufport_fill(ScmPort *p, int min, int allow_less)
         p->src.buf.current = p->src.buf.end = p->src.buf.buffer;
     }
     if (min <= 0) min = SCM_PORT_BUFFER_ROOM(p);
-    if (p->src.buf.mode == SCM_PORT_BUFFER_FULL) {
+    if (p->src.buf.mode != SCM_PORT_BUFFER_NONE) {
         toread = SCM_PORT_BUFFER_ROOM(p);
     } else {
         toread = min;
@@ -609,9 +612,10 @@ static int bufport_read(ScmPort *p, char *dst, int siz)
 #define PORT_VECTOR_SIZE 256    /* need to be 2^n */
 
 static struct {
+    int dummy;
     ScmWeakVector   *ports;
     ScmInternalMutex mutex;
-} active_buffered_ports = { NULL };
+} active_buffered_ports = { 1, NULL }; /* magic to put this in .data area */
 
 #define PORT_HASH(port)  \
     ((((SCM_WORD(port)>>3) * 2654435761UL)>>16) % PORT_VECTOR_SIZE)
@@ -684,7 +688,7 @@ void Scm_FlushAllPorts(int exitting)
         (void)SCM_INTERNAL_MUTEX_UNLOCK(active_buffered_ports.mutex);
         if (!SCM_FALSEP(p)) {
             SCM_ASSERT(SCM_PORTP(p) && SCM_PORT_TYPE(p)==SCM_PORT_FILE);
-            if (!SCM_PORT_ERROR_P(SCM_PORT(p))) {
+            if (!SCM_PORT_ERROR_OCCURRED_P(SCM_PORT(p))) {
                 bufport_flush(SCM_PORT(p), 0, TRUE);
             }
         }
@@ -800,11 +804,11 @@ static int file_flusher(ScmPort *p, int cnt, int forcep)
     return nwrote;
 }
 
-static int file_closer(ScmPort *p)
+static void file_closer(ScmPort *p)
 {
     int fd = (int)p->src.buf.data;
     SCM_ASSERT(fd >= 0);
-    return close(fd);
+    close(fd);
 }
 
 static int file_ready(ScmPort *p)
@@ -833,8 +837,9 @@ ScmObj Scm_OpenFilePort(const char *path, int flags, int buffering, int perm)
     if ((flags & O_ACCMODE) == O_RDONLY) dir = SCM_PORT_INPUT;
     else if ((flags & O_ACCMODE) == O_WRONLY) dir = SCM_PORT_OUTPUT;
     else Scm_Error("unsupported file access mode %d to open %s", flags&O_ACCMODE, path);
-    if (buffering < SCM_PORT_BUFFER_FULL || buffering > SCM_PORT_BUFFER_NONE)
+    if (buffering < SCM_PORT_BUFFER_FULL || buffering > SCM_PORT_BUFFER_NONE) {
         Scm_Error("bad buffering flag: %d", buffering);
+    }
     fd = open(path, flags, perm);
     if (fd < 0) return SCM_FALSE;
     bufrec.mode = buffering;
@@ -847,7 +852,8 @@ ScmObj Scm_OpenFilePort(const char *path, int flags, int buffering, int perm)
     bufrec.filenum = file_filenum;
     bufrec.seeker = file_seeker;
     bufrec.data = (void*)fd;
-    p = Scm_MakeBufferedPort(SCM_MAKE_STR_COPYING(path), dir, TRUE, &bufrec);
+    p = Scm_MakeBufferedPort(SCM_CLASS_PORT, SCM_MAKE_STR_COPYING(path),
+                             dir, TRUE, &bufrec);
     return p;
 }
 
@@ -875,7 +881,7 @@ ScmObj Scm_MakePortWithFd(ScmObj name, int direction,
     bufrec.seeker = NULL;
     bufrec.data = (void*)fd;
     
-    p = Scm_MakeBufferedPort(name, direction, ownerp, &bufrec);
+    p = Scm_MakeBufferedPort(SCM_CLASS_PORT, name, direction, ownerp, &bufrec);
     return p;
 }
 
@@ -885,21 +891,21 @@ ScmObj Scm_MakePortWithFd(ScmObj name, int direction,
 
 ScmObj Scm_MakeInputStringPort(ScmString *str, int privatep)
 {
-    ScmPort *p = make_port(SCM_PORT_INPUT, SCM_PORT_ISTR);
+    ScmPort *p = make_port(SCM_CLASS_PORT, SCM_PORT_INPUT, SCM_PORT_ISTR);
     p->src.istr.start = SCM_STRING_START(str);
     p->src.istr.current = SCM_STRING_START(str);
     p->src.istr.end = SCM_STRING_START(str) + SCM_STRING_SIZE(str);
     SCM_PORT(p)->name = SCM_MAKE_STR("(input string port)");
-    if (privatep) PORT_LOCK(p, Scm_VM());
+    if (privatep) PORT_PRELOCK(p, Scm_VM());
     return SCM_OBJ(p);
 }
 
 ScmObj Scm_MakeOutputStringPort(int privatep)
 {
-    ScmPort *p = make_port(SCM_PORT_OUTPUT, SCM_PORT_OSTR);
+    ScmPort *p = make_port(SCM_CLASS_PORT, SCM_PORT_OUTPUT, SCM_PORT_OSTR);
     Scm_DStringInit(&p->src.ostr);
     SCM_PORT(p)->name = SCM_MAKE_STR("(output string port)");
-    if (privatep) PORT_LOCK(p, Scm_VM());
+    if (privatep) PORT_PRELOCK(p, Scm_VM());
     return SCM_OBJ(p);
 }
 
@@ -939,6 +945,12 @@ ScmObj Scm_GetRemainingInputString(ScmPort *port)
  * Procedural port
  */
 
+/* To create a procedural port, fill in the ScmPortVTable function
+   pointers and pass it to Scm_MakeVirutalPort.  You don't need to
+   provide all the functions; put NULL if you think you don't
+   provide the functionality.
+*/
+
 /* default dummy procedures */
 static int null_getb(ScmPort *dummy)
     /*ARGSUSED*/
@@ -958,57 +970,47 @@ static int null_getz(char *buf, int buflen, ScmPort *dummy)
     return 0;
 }
 
-static ScmObj null_getline(ScmPort *port)
-{
-    return SCM_EOF;
-}
-
-static int null_ready(ScmPort *dummy)
+static int null_ready(ScmPort *dummy, int charp)
     /*ARGSUSED*/
 {
     return TRUE;
 }
 
-static int null_putb(ScmByte b, ScmPort *dummy)
+static void null_putb(ScmByte b, ScmPort *dummy)
     /*ARGSUSED*/
 {
-    return 0;
 }
 
-static int null_putc(ScmChar c, ScmPort *dummy)
+static void null_putc(ScmChar c, ScmPort *dummy)
     /*ARGSUSED*/
 {
-    return 0;
 }
 
-static int null_putz(const char *str, int len, ScmPort *dummy)
+static void null_putz(const char *str, int len, ScmPort *dummy)
     /*ARGSUSED*/
 {
-    return 0;
 }
 
-static int null_puts(ScmString *s, ScmPort *dummy)
+static void null_puts(ScmString *s, ScmPort *dummy)
     /*ARGSUSED*/
 {
-    return 0;
 }
 
-static int null_flush(ScmPort *dummy)
+static void null_flush(ScmPort *dummy)
     /*ARGSUSED*/
 {
-    return 0;
 }
 
-ScmObj Scm_MakeVirtualPort(int direction, ScmPortVTable *vtable)
+ScmObj Scm_MakeVirtualPort(ScmClass *klass, int direction,
+                           ScmPortVTable *vtable)
 {
-    ScmPort *p = make_port(direction, SCM_PORT_PROC);
+    ScmPort *p = make_port(klass, direction, SCM_PORT_PROC);
     
     /* Copy vtable, and ensure all entries contain some ptr */
     p->src.vt = *vtable;
     if (!p->src.vt.Getb)  p->src.vt.Getb = null_getb;
     if (!p->src.vt.Getc)  p->src.vt.Getc = null_getc;
     if (!p->src.vt.Getz)  p->src.vt.Getz = null_getz;
-    if (!p->src.vt.Getline) p->src.vt.Getline = null_getline;
     if (!p->src.vt.Ready) p->src.vt.Ready = null_ready;
     if (!p->src.vt.Putb)  p->src.vt.Putb = null_putb;
     if (!p->src.vt.Putc)  p->src.vt.Putc = null_putc;
@@ -1019,6 +1021,252 @@ ScmObj Scm_MakeVirtualPort(int direction, ScmPortVTable *vtable)
     return SCM_OBJ(p);
 }
 
+/*===============================================================
+ * Coding-aware port
+ */
+
+/* Coding-aware port wraps an input port, and specifically recognizes
+   'coding' magic comment.   It is primarily used when loading source
+   code, but can be used separately. */
+
+/* gauche.charconv sets the pointer */
+ScmPort *(*Scm_CodingAwarePortHook)(ScmPort *src,
+                                    const char *srcencoding)
+    = NULL;
+
+#define CODING_MAGIC_COMMENT_LINES 2 /* maximum number of lines to be
+                                        looked at for the 'encoding' magic
+                                        comment. */
+
+typedef struct coding_port_data_rec {
+    ScmPort *source;            /* source port */
+    int state;                  /* port state; see below */
+    const char *pbuf;           /* prefetched buffer.  NUL terminated.
+                                   contains at most CODING_MAGIC_COMMENT_LINES
+                                   newlines. */
+    int pbufsize;               /* # of bytes in pbuf */
+} coding_port_data;
+
+enum {
+    CODING_PORT_INIT,           /* initial state */
+    CODING_PORT_RECOGNIZED,     /* prefetched up to two lines, and
+                                   conversion port is set if necessary.
+                                   there are buffered data in lines[]. */
+    CODING_PORT_FLUSHED         /* prefetched lines are flushed. */
+};
+
+/* A hardcoded DFA to recognize #/;.*coding[:=]\s*([\w.-]+)/ */
+static const char *look_for_encoding(const char *buf)
+{
+    char c;
+    const char *s;
+    char *encoding;
+    
+  init:
+    for (;;) {
+        switch (*buf++) {
+        case '\0': return NULL;
+        case ';':  goto comment;
+        }
+    }
+  comment:
+    for (;;) {
+        switch (*buf++) {
+        case '\0': return NULL;
+        case '\n': goto init;
+        case '\r': if (*buf != '\n') goto init; break;
+        case 'c' : goto coding;
+        }
+    }
+  coding:
+    if (strncmp(buf, "oding", 5) != 0) goto comment;
+    buf+=5;
+    if (*buf != ':' && *buf != '=') goto comment;
+    for (buf++;;buf++) {
+        if (*buf != ' ' && *buf != '\t') break;
+    }
+    if (*buf == '\0') return NULL;
+
+    for (s = buf;*buf;buf++) {
+        if (!isalnum(*buf) && *buf != '_' && *buf != '-' && *buf != '.') {
+            break;
+        }
+    }
+    if (s == buf) goto comment;
+
+    /* Here we found a matching string, starting from s and ends at buf. */
+
+    /* kludge: Emacs uses special suffix #/-(unix|dos|mac)$/ to distinguish
+       EOL variants.  For compatibility, drop such suffix if we have one. */
+    if (buf-s > 5 && (strncmp(buf-5, "-unix", 5) == 0)) {
+        buf -= 5;
+    } else if (buf-s > 4 && (strncmp(buf-4, "-dos", 4) == 0
+                             || strncmp(buf-4, "-mac", 4) == 0)) {
+        buf -= 4;
+    }
+
+    /* Copy and return the encoding string */
+    encoding = SCM_NEW_ATOMIC2(char*, buf-s+1);
+    memcpy(encoding, s, buf-s);
+    encoding[buf-s] = '\0';
+    return encoding;
+}
+
+static void coding_port_recognize_encoding(ScmPort *port,
+                                           coding_port_data *data)
+{
+    ScmDString ds;
+    int num_newlines = 0, c;
+    int cr_seen = FALSE;
+    const char *encoding = NULL;
+
+    SCM_ASSERT(data->source != NULL);
+
+    /* Prefetch up to CODING_MAGIC_COMMENT_LINES lines or the first NUL
+       character.   data->pbuf ends up holding NUL terminated string. */
+    Scm_DStringInit(&ds);
+    for (;;) {
+        c = Scm_GetbUnsafe(data->source);
+        if (c == EOF) break;
+        if (c == 0) {
+            /* take extra care not to lose '\0' */
+            Scm_UngetbUnsafe(c, data->source);
+            break;
+        }
+        SCM_DSTRING_PUTB(&ds, c);
+        if (c == '\r') {   /* for the source that only uses '\r' */
+            cr_seen = TRUE;
+        } else if (c == '\n' || cr_seen) {
+            if (++num_newlines >= CODING_MAGIC_COMMENT_LINES) {
+                break;
+            }
+        } else {
+            cr_seen = FALSE;
+        }
+    }
+    data->pbuf = Scm_DStringGetz(&ds);
+    data->pbufsize = strlen(data->pbuf);
+    
+    /* Look for the magic comment */
+    encoding = look_for_encoding(data->pbuf);
+
+    /* Wrap the source port by conversion port, if necessary. */
+    if (encoding == NULL || Scm_SupportedCharacterEncodingP(encoding)) {
+        return;
+    }
+
+    if (Scm_CodingAwarePortHook == NULL) {
+        /* Require gauche.charconv.
+           NB: we don't need mutex here, for loading the module is
+           serialized in Scm_Require. */
+        Scm_Require(SCM_MAKE_STR("gauche/charconv"));
+        if (Scm_CodingAwarePortHook == NULL) {
+            Scm_PortError(port, SCM_PORT_ERROR_OTHER,
+                          "couldn't load gauche.charconv module");
+        }
+    }
+    data->source = Scm_CodingAwarePortHook(data->source, encoding);
+}
+
+static int coding_filler(ScmPort *p, int cnt)
+{
+    int nread = 0, i;
+    coding_port_data *data = (coding_port_data*)p->src.buf.data;
+    char *datptr = p->src.buf.end;
+
+    SCM_ASSERT(data->source);
+
+    /* deals with the most frequent case */
+    if (data->state == CODING_PORT_FLUSHED) {
+        return Scm_GetzUnsafe(datptr, cnt, data->source);
+    }
+    
+    if (data->state == CODING_PORT_INIT) {
+        coding_port_recognize_encoding(p, data);
+        data->state = CODING_PORT_RECOGNIZED;
+    }
+
+    /* Here, we have data->state == CODING_PORT_RECOGNIZED */
+    if (data->pbufsize > 0) {
+        if (data->pbufsize <= cnt) {
+            memcpy(datptr, data->pbuf, data->pbufsize);
+            nread = data->pbufsize;
+            data->pbuf = NULL;
+            data->pbufsize = 0;
+            data->state = CODING_PORT_FLUSHED;
+        } else {
+            memcpy(datptr, data->pbuf, cnt);
+            nread = cnt;
+            data->pbuf += cnt;
+            data->pbufsize -= cnt;
+        }
+        return nread;
+    } else {
+        data->state = CODING_PORT_FLUSHED;
+        return Scm_GetzUnsafe(datptr, cnt, data->source);
+    }
+}
+
+static void coding_closer(ScmPort *p)
+{
+    coding_port_data *data = (coding_port_data*)p->src.buf.data;
+    if (data->source) {
+        Scm_ClosePort(data->source);
+        data->source = NULL;
+    }
+}
+
+static int coding_ready(ScmPort *p)
+{
+    coding_port_data *data = (coding_port_data*)p->src.buf.data;
+    if (data->source == NULL) return TRUE;
+    if (data->state == CODING_PORT_RECOGNIZED) {
+        return SCM_FD_READY;
+    } else {
+        return Scm_ByteReadyUnsafe(p);
+    }
+}
+
+static int coding_filenum(ScmPort *p)
+{
+    coding_port_data *data = (coding_port_data*)p->src.buf.data;
+    if (data->source == NULL) return -1;
+    return Scm_PortFileNo(data->source);
+}
+
+ScmObj Scm_MakeCodingAwarePort(ScmPort *iport)
+{
+    ScmObj p;
+    ScmPortBuffer bufrec;
+    coding_port_data *data;
+    int i;
+
+    if (!SCM_IPORTP(iport)) {
+        Scm_Error("open-coding-aware-port requires an input port, but got %S", iport);
+    }
+    data = SCM_NEW(coding_port_data);
+    data->source = iport;
+    data->state = CODING_PORT_INIT;
+    data->pbuf = NULL;
+    data->pbufsize = 0;
+
+    bufrec.mode = SCM_PORT_BUFFER_FULL;
+    bufrec.buffer = NULL;
+    bufrec.size = 0;
+    bufrec.filler = coding_filler;
+    bufrec.flusher = NULL;
+    bufrec.closer = coding_closer;
+    bufrec.ready = coding_ready;
+    bufrec.filenum = coding_filenum;
+    bufrec.seeker = NULL;
+    bufrec.data = (void*)data;
+    p = Scm_MakeBufferedPort(SCM_CLASS_CODING_AWARE_PORT,
+                             Scm_PortName(iport), SCM_PORT_INPUT,
+                             TRUE, &bufrec);
+    return p;
+}
+
+
 /*===============================================================
  * with-port
  */
@@ -1110,8 +1358,10 @@ void Scm__InitPort(void)
     (void)SCM_INTERNAL_MUTEX_INIT(active_buffered_ports.mutex);
     active_buffered_ports.ports = SCM_WEAKVECTOR(Scm_MakeWeakVector(PORT_VECTOR_SIZE));
 
-    Scm_InitBuiltinClass(&Scm_PortClass, "<port>", NULL, TRUE,
-                         Scm_GaucheModule());
+    Scm_InitStaticClass(&Scm_PortClass, "<port>",
+                        Scm_GaucheModule(), NULL, 0);
+    Scm_InitStaticClass(&Scm_CodingAwarePortClass, "<coding-aware-port>",
+                        Scm_GaucheModule(), NULL, 0);
 
     scm_stdin  = Scm_MakePortWithFd(SCM_MAKE_STR("(stdin)"),
                                     SCM_PORT_INPUT, 0,
