@@ -30,7 +30,7 @@
  *   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  *   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- *  $Id: read.c,v 1.70 2004/02/24 09:08:46 shirok Exp $
+ *  $Id: read.c,v 1.75 2004/10/10 05:34:30 shirok Exp $
  */
 
 #include <stdio.h>
@@ -190,6 +190,7 @@ void Scm_ReadError(ScmPort *port, const char *msg, ...)
 {
     ScmObj ostr = Scm_MakeOutputStringPort(TRUE);
     ScmObj name = Scm_PortName(port);
+    ScmObj rerr;
     int line = Scm_PortLine(port);
     va_list ap;
 
@@ -201,7 +202,9 @@ void Scm_ReadError(ScmPort *port, const char *msg, ...)
     va_start(ap, msg);
     Scm_Vprintf(SCM_PORT(ostr), msg, ap, TRUE);
     va_end(ap);
-    Scm_Error("%A", Scm_GetOutputString(SCM_PORT(ostr)));
+
+    rerr = Scm_MakeReadError(Scm_GetOutputString(SCM_PORT(ostr)), port, line);
+    Scm_VMThrowException(rerr);
 }
 
 /*----------------------------------------------------------------
@@ -360,7 +363,7 @@ static void read_nested_comment(ScmPort *port, ScmReadContext *ctx)
             continue;
         case EOF:
           eof:
-            Scm_Error("encountered EOF inside nested multi-line comment (comment begins at line %d)", line);
+            Scm_ReadError(port, "encountered EOF inside nested multi-line comment (comment begins at line %d)", line);
         default:
             break;
         }
@@ -379,7 +382,11 @@ static int skipws(ScmPort *port, ScmReadContext *ctx)
                 /* NB: comment may contain unexpected character code.
                    for the safety, we read bytes here. */
                 c = Scm_GetbUnsafe(port);
-                if (c == '\n') break;
+                if (c == '\n') {
+                    /* oops.  ugly. */
+                    port->line++;
+                    break;
+                }
                 if (c == EOF) return EOF;
             }
             continue;
@@ -481,6 +488,10 @@ static ScmObj read_internal(ScmPort *port, ScmReadContext *ctx)
                     if (c2 == '"') return read_string(port, TRUE, ctx);
                     Scm_ReadError(port, "unsupported #*-syntax: #*%C", c2);
                 }
+            case ';':
+                /* #;expr - comment out sexpr */
+                read_item(port, ctx); /* read and discard */
+                return SCM_UNDEFINED; /* indicate this is a comment */
             default:
                 Scm_ReadError(port, "unsupported #-syntax: #%C", c1);
             }
@@ -678,8 +689,9 @@ static ScmChar read_string_xdigits(ScmPort *port, int ndigs, int key,
         Scm_DStringPutc(&ds, '\\');
         Scm_DStringPutc(&ds, key);
         for (i=0; i<nread; i++) Scm_DStringPutc(&ds, (unsigned char)buf[i]);
-        Scm_Error("Bad '\\%c' escape sequence in a string literal: %s",
-                  key, Scm_DStringGetz(&ds));
+        Scm_ReadError(port,
+                      "Bad '\\%c' escape sequence in a string literal: %s",
+                      key, Scm_DStringGetz(&ds));
     }
     return r;
 }
@@ -1122,7 +1134,7 @@ static ScmObj maybe_uvector(ScmPort *port, char ch, ScmReadContext *ctx)
            serialized in Scm_Require. */
         Scm_Require(SCM_MAKE_STR("gauche/uvector"));
         if (Scm_ReadUvectorHook == NULL)
-            Scm_Error("couldn't load srfi-4 module");
+            Scm_ReadError(port, "couldn't load srfi-4 module");
     }
     return Scm_ReadUvectorHook(port, tag, ctx);
 }
