@@ -30,7 +30,7 @@
  *   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  *   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- *  $Id: vm.c,v 1.206 2004/01/18 12:07:31 shirok Exp $
+ *  $Id: vm.c,v 1.217 2004/11/23 13:10:00 shirok Exp $
  */
 
 #define LIBGAUCHE_BODY
@@ -82,6 +82,7 @@ static ScmSubr default_exception_handler_rec;
 #define DEFAULT_EXCEPTION_HANDLER  SCM_OBJ(&default_exception_handler_rec)
 static ScmObj throw_cont_calculate_handlers(ScmEscapePoint *, ScmVM *);
 static ScmObj throw_cont_body(ScmObj, ScmEscapePoint*, ScmObj);
+static void   process_queued_requests(ScmVM *vm);
 
 /*
  * Constructor
@@ -116,6 +117,7 @@ ScmVM *Scm_NewVM(ScmVM *base,
 
     v->compilerFlags = base? base->compilerFlags : 0;
     v->runtimeFlags = base? base->runtimeFlags : 0;
+    v->queueNotEmpty = 0;
 
     v->stack = SCM_NEW_ARRAY(ScmObj, SCM_VM_STACK_SIZE);
     v->sp = v->stack;
@@ -143,11 +145,8 @@ ScmVM *Scm_NewVM(ScmVM *base,
     v->load_next = SCM_NIL;
     v->load_port = SCM_FALSE;
 
-    for (i=0; i<SCM_VM_SIGQ_SIZE; i++) v->sigQueue[i] = -1;
-    v->sigQueueHead = v->sigQueueTail = 0;
-    v->sigOverflow = 0;
-    v->sigPending = SCM_NIL;
     sigemptyset(&v->sigMask);
+    Scm_SignalQueueInit(&v->sigq);
 
     return v;
 }
@@ -450,7 +449,17 @@ static void run_loop()
     
     for (;;) {
         /*VM_DUMP("");*/
-        SCM_SIGCHECK(vm);
+        
+        /* Check if there's a queued processing first. */
+        if (vm->queueNotEmpty) {
+            CHECK_STACK(CONT_FRAME_SIZE);
+            PUSH_CONT(prevpc, pc);
+            SAVE_REGS();
+            process_queued_requests(vm);
+            RESTORE_REGS();
+            POP_CONT();
+            continue;
+        }
 
         /* See if we're at the end of procedure.  It's safer to use
            !SCM_PAIRP(pc) than SCM_NULLP(pc), but the latter is faster.
@@ -599,30 +608,34 @@ static void run_loop()
                 /* First, compute methods */
                 nm = SCM_FALSE;
                 if (proctype == SCM_PROC_GENERIC) {
-                    if (SCM_GENERICP(val0)) {
-                      generic:
-                        /* pure generic application */
-                        mm = Scm_ComputeApplicableMethods(SCM_GENERIC(val0),
-                                                          argp, argc);
-                        if (!SCM_NULLP(mm)) {
-                            mm = Scm_SortMethods(mm, argp, argc);
-                            nm = Scm_MakeNextMethod(SCM_GENERIC(val0),
-                                                    SCM_CDR(mm),
-                                                    argp, argc, TRUE);
-                            val0 = SCM_CAR(mm);
-                            proctype = SCM_PROC_METHOD;
-                        }
-                    } else {
-                        /* use scheme-defined MOP */
+                    if (!SCM_GENERICP(val0)) {
+                        /* use scheme-defined MOP.  we modify the stack frame
+                           so that it is converted to an application of
+                           pure generic fn apply-generic. */
                         ScmObj args = SCM_NIL, arg;
                         int i;
                         for (i=0; i<argc; i++) {
                             POP_ARG(arg);
                             args = Scm_Cons(arg, args);
                         }
-                        Scm_VMApply2(SCM_OBJ(&Scm_GenericApplyGeneric),
-                                     val0, args);
-                        NEXT;
+                        argp = sp;
+                        argc = 2;
+                        CHECK_STACK(2);
+                        PUSH_ARG(val0);
+                        PUSH_ARG(args);
+                        val0 = SCM_OBJ(&Scm_GenericApplyGeneric);
+                    }
+                  generic:
+                    /* pure generic application */
+                    mm = Scm_ComputeApplicableMethods(SCM_GENERIC(val0),
+                                                      argp, argc);
+                    if (!SCM_NULLP(mm)) {   
+                        mm = Scm_SortMethods(mm, argp, argc);
+                        nm = Scm_MakeNextMethod(SCM_GENERIC(val0),
+                                                SCM_CDR(mm),
+                                                argp, argc, TRUE);
+                        val0 = SCM_CAR(mm);
+                        proctype = SCM_PROC_METHOD;
                     }
                 } else if (proctype == SCM_PROC_NEXT_METHOD) {
                     ScmNextMethod *n = SCM_NEXT_METHOD(val0);
@@ -1988,16 +2001,39 @@ ScmObj Scm_VMDynamicWindC(ScmObj (*before)(ScmObj *args, int nargs, void *data),
 
 /* Conceptually, exception handling is nothing more than a particular
  * combination of dynamic-wind and call/cc.   Gauche implements a parts
- * of it so that it will be efficient and safer to use.
+ * of it in C so that it will be efficient and safer to use.
  *
- * The most basic layer consists of those three functions (SRFI-18).
+ * The most basic layer consists of these two functions:
  *
- *  current-exception-handler
  *  with-exception-handler
  *  raise
  *
- * Their behavior is explained well in the following Scheme code,
- * if we ignore the messy part to keep C stack sane.
+ * There is a slight problem, though.  These two functions are defined
+ * both in srfi-18 (multithreads) and srfi-34 (exception handling), and
+ * two disagrees in the semantics of raise.
+ *
+ * Srfi-18 requires an exception handler to be called with the same dynamic
+ * environment as the one of the primitive that raises the exception.
+ * That means when an exception handler is running, the current
+ * exception handler is the running handler itself.  Naturally, calling
+ * raise unconditionally within the exception handler causes infinite loop.
+ *
+ * Srfi-34 says that an exception handler is called with the same dynamic
+ * envionment where the exception is raised, _except_ that the current
+ * exception handler is "popped", i.e. when an exception handler is running,
+ * the current exception handler is the "outer" or "old" one.  Calling
+ * raise within an exception handler passes the control to the outer
+ * exception handler.
+ *
+ * At this point I haven't decided which model Gauche should support natively.
+ * The current implementation predates srfi-34 and roughly follows srfi-18.
+ * It appears that srfi-18's mechanism is more "primitive" or "lightweight"
+ * than srfi-34's, so it's likely that Gauche will continue to support
+ * srfi-18 model natively, and maybe provides srfi-34's interface by an
+ * additional module.
+ *
+ * The following is a model of the current implementation, sans the messy
+ * part of handling C stacks.
  * Suppose a system variable %xh keeps the list of exception handlers.
  *
  *  (define (current-exception-handler) (car %xh))
@@ -2022,8 +2058,9 @@ ScmObj Scm_VMDynamicWindC(ScmObj (*before)(ScmObj *args, int nargs, void *data),
  * Note that this model assumes an exception handler returns unless it
  * explictly invokes continuation captured elsewhere.   In reality,
  * "error" exceptions are not supposed to return (hence it is checked
- * in raise).  Gauche provides two more useful exception handling
- * constructs that automates such continuation capturing.
+ * in raise).  Gauche provides another useful exception handling
+ * constructs that automates such continuation capturing.  It can be
+ * explained by the following code.
  *
  * (define (with-error-handler handler thunk)
  *   (call/cc
@@ -2123,10 +2160,11 @@ static SCM_DEFINE_SUBR(default_exception_handler_rec, 1, 0,
 /*
  * Entry point of throwing exception.
  *
- *  This function may be called from Scheme function raise or throw,
- *  or C-function Scm_Error families and signal handler.   So we can't
- *  push the handler to continuation and return; we have to use Scm_Apply
- *  to run the handler.
+ *  This function can be called from Scheme function raise,
+ *  or C-function Scm_Error families and signal handler. 
+ *  So there may be a raw C code in the continuation of this C call.
+ *  Thus we can't use Scm_VMApply to call the user-defined exception
+ *  handler.
  */
 ScmObj Scm_VMThrowException(ScmObj exception)
 {
@@ -2137,7 +2175,7 @@ ScmObj Scm_VMThrowException(ScmObj exception)
 
     if (vm->exceptionHandler != DEFAULT_EXCEPTION_HANDLER) {
         vm->val0 = Scm_Apply(vm->exceptionHandler, SCM_LIST1(exception));
-        if (SCM_ERRORP(exception)) {
+        if (SCM_SERIOUS_CONDITION_P(exception)) {
             /* the user-installed exception handler returned while it
                shouldn't.  In order to prevent infinite loop, we should
                pop the erroneous handler.  For now, we just reset
@@ -2146,7 +2184,7 @@ ScmObj Scm_VMThrowException(ScmObj exception)
             Scm_Error("user-defined exception handler returned on non-continuable exception %S", exception);
         }
         return vm->val0;
-    } else if (!SCM_ERRORP(exception)) {
+    } else if (!SCM_SERIOUS_CONDITION_P(exception)) {
         /* The system's default handler does't care about
            continuable exception.  See if there's a user-defined
            exception handler in the chain.  */
@@ -2434,6 +2472,72 @@ ScmObj Scm_Values5(ScmObj val0, ScmObj val1, ScmObj val2, ScmObj val3, ScmObj va
     return val0;
 }
 
+/*==================================================================
+ * Queued handler processing.
+ */
+
+/* Signal handlers and finalizers are queued in VM when they
+ * are requested, and processed when VM is in consistent state.
+ * process_queued_requests() are called near the beginning of
+ * VM loop, when the VM checks if there's any queued request.
+ *
+ * When this procedure is called, VM is in middle of any two
+ * VM instructions.  We need to make sure the handlers won't
+ * disturb the VM state.
+ *
+ * Conceptually, this procedure inserts handler invocations before
+ * the current continuation.
+ */
+
+static ScmObj process_queued_requests_cc(ScmObj result, void **data)
+{
+    /* restore the saved continuation of normal execution flow of VM */
+    int i;
+    ScmObj cp;
+    ScmVM *vm = theVM;
+    vm->numVals = (int)data[0];
+    vm->val0 = data[1];
+    if (vm->numVals > 1) {
+        cp = SCM_OBJ(data[2]);
+        for (i=0; i<vm->numVals-1; i++) {
+            vm->vals[i] = SCM_CAR(cp);
+            cp = SCM_CDR(cp);
+        }
+    }
+    return vm->val0;
+}
+
+static void process_queued_requests(ScmVM *vm)
+{
+    void *data[3];
+
+    /* preserve the current continuation */
+    data[0] = (void*)vm->numVals;
+    data[1] = vm->val0;
+    if (vm->numVals > 1) {
+        int i;
+        ScmObj h = SCM_NIL, t = SCM_NIL;
+
+        for (i=0; i<vm->numVals-1; i++) {
+            SCM_APPEND1(h, t, vm->vals[i]);
+        }
+        data[2] = h;
+    } else {
+        data[2] = NULL;
+    }
+    Scm_VMPushCC(process_queued_requests_cc, data, 3);
+
+    /* Process queued stuff.  Currently they call VM recursively,
+       but we'd better to arrange them to be processed in the same
+       VM level. */
+    if (vm->queueNotEmpty & SCM_VM_SIGQ_MASK) {
+        Scm_SigCheck(vm);
+    }
+    if (vm->queueNotEmpty & SCM_VM_FINQ_MASK) {
+        Scm_VMFinalizerRun(vm);
+    }
+}
+
 /*==============================================================
  * Debug features.
  */
