@@ -53,7 +53,7 @@
     (begin
       (with-output-to-file "test.out/test6.o" (lambda () (newline)))
       (with-output-to-file "test.out/test7.o" (lambda () (newline)))
-      (sys-mkdir "test.out/test2.d")))
+      (sys-mkdir "test.out/test2.d" #o777)))
 
 (sys-system "mkdir test.out/test.d")
 (with-output-to-file "test.out/test.d/test10.o"
@@ -200,6 +200,9 @@
 (test* "build-path" "/a/b/c" (build-path "/" "a/b" "c"))
 (test* "build-path" "./a/b/c" (build-path "." "a/b" "c"))
 (test* "build-path" *test-error* (build-path "." "/a/b"))
+(test* "build-path" "foo" (build-path "" "foo"))
+(test* "build-path" "foo/bar" (build-path "" "foo" "" "bar"))
+(test* "build-path" "foo" (build-path "" "foo" ""))
 
 (test* "resolve-path" "/" (resolve-path "/"))
 (test* "resolve-path" "." (resolve-path "."))
