@@ -9,7 +9,7 @@
 ;;-------------------------------------------------------------------
 (test-section "file i/o")
 
-(sys-system "rm -f tmp2.o")
+(sys-system "rm -rf tmp2.o")
 
 (test* "open-input-file" *test-error*
        (open-input-file "tmp2.o"))
@@ -280,7 +280,32 @@
          (lambda (p)
            (read-char p)
            (port-seek p -2))))
+;; ungetc and seek interaction; pointed out by Alex Shinn
+(test* "seek (istr, with peek-char)" '("hello" "hello")
+       (letrec ((read-zstring
+                 (lambda (p)
+                   (let loop ((ls '()))
+                     (let ((c (peek-char p)))
+                       (if (or (eof-object? c) (eqv? c #\null))
+                         (list->string (reverse ls))
+                         (begin (read-char p) (loop (cons c ls)))))))))
+         (call-with-input-string "hello\0world"
+           (lambda (p)
+             (let* ((first (read-zstring p))
+                    (dummy (port-seek p 0))
+                    (second (read-zstring p)))
+               (list first second))))))
+(test* "seek (istr, with peek-char)" '(#\b #\b)
+       (with-input-from-string "abc"
+         (lambda ()
+           (read-char)
+           (let ((c1 (peek-char)))
+             (port-seek (current-input-port) 0 SEEK_CUR)
+             (list c1 (peek-char))))))
 
+;; NB: in the following four test, each ifile-ofile test is a pair
+;;     (the ofile test depends on the previous state by ifile).  do not
+;;     separate them.
 (test* "seek (ifile)" "abcdecdefgfghijabchij"
        (begin
          (sys-unlink "test.o")
@@ -360,6 +385,25 @@
                  (display (read-block 4 p))))))
          ))
 
+(test* "seek (ifile, with peek-char)" '("hello" "hello")
+       (letrec ((read-zstring
+                 (lambda (p)
+                   (let loop ((ls '()))
+                     (let ((c (peek-char p)))
+                       (if (or (eof-object? c) (eqv? c #\null))
+                         (list->string (reverse ls))
+                         (begin (read-char p) (loop (cons c ls)))))))))
+         (begin
+           (sys-unlink "test.o")
+           (with-output-to-file "test.o"
+             (lambda () (display "hello\0world")))
+           (call-with-input-file "test.o"
+             (lambda (p)
+               (let* ((first (read-zstring p))
+                      (dummy (port-seek p 0))
+                      (second (read-zstring p)))
+                 (list first second)))))))
+
 (sys-unlink "test.o")
 
 ;;-------------------------------------------------------------------
@@ -526,5 +570,92 @@
  \"#| this is a string, not a comment\"
  |# and this is not a comment terminator but an escaped symbol|)"))
 
+;;-------------------------------------------------------------------
+(test-section "#;-style commend (a la Chez)")
+
+(test* "#;123 456" 456
+       (read-from-string "#;123 456"))
+(test* "#; (123 456) 789" 789
+       (read-from-string "#; (123 456) 789"))
+(test* "#;#;(123 456) 789 1" 1
+       (read-from-string "#;#;(123 456) 789 1"))
+
+;;-------------------------------------------------------------------
+(test-section "coding-aware-port basic")
+
+;; Testing source port _without_ any conversion.  Basically, these
+;; tests just checks up the boundary condition of source-port prefetching
+;; routine.
+;; The actual conversion is tested in ext/charconv.
+
+(define (with-coding-aware-port input proc)
+  (let* ((src  (open-input-string input))
+         (wrap (open-coding-aware-port src)))
+    (proc src wrap)))
+
+(test* "ownership" #t
+       (with-coding-aware-port
+        "abc"
+        (lambda (src wrap)       
+          (close-input-port wrap)          
+          (port-closed? src))))
+
+(test* "read from empty port" '(#t #t #t #t #t)
+       (map (lambda (p) (with-coding-aware-port "" p))
+            (list
+             (lambda (src wrap) (eof-object? (read-char wrap)))
+             (lambda (src wrap) (eof-object? (read-byte wrap)))
+             (lambda (src wrap) (eof-object? (peek-char wrap)))
+             (lambda (src wrap) (eof-object? (peek-byte wrap)))
+             (lambda (src wrap) (eof-object? (read-line wrap))))))
+
+(let ((tdata '("abc" "abc\n" "\nabc" "abc\ndef\n"
+               "abc\ndef\nghi" "abc\ndef\nghi\n"
+               "abc\r\ndef\r\nghi" "abc\r\ndef\r\nghi\r\n"
+               "abc\rdef\rghi" "abc\rdef\rghi\r"
+               "abc\ndef\nghi\njkl" "abc\ndef\nghi\njkl\n")))
+  (test* "read from simple contents"
+         tdata
+         (map (lambda (i)
+                (with-coding-aware-port
+                 i
+                 (lambda (src wrap)
+                   (let loop ((ch (read-char wrap))
+                              (r  '()))
+                     (if (eof-object? ch)
+                       (list->string (reverse r))
+                       (loop (read-char wrap) (cons ch r)))))))
+              tdata)))
+
+(let ((tdata '("coding: abcde\naa"
+               ";coding:\nabcdef\naa"
+               ";coding:\r\nabcdef\r\naa"
+               ";coding:\rabcdef\raa"
+               "coding: coding: coding:; abcde\naa"
+               "coding: coding: coding:; abcde\r\naa"
+               ";; co\nding: foobar\naa"
+               ";; co\r\nding: foobar\r\naa"
+               ";; co\rding: foobar\raa"
+               ";; coding:\n;; foobar\naa"
+               ";; coding : foobar\naa"
+               ";; coding : foobar\r\naa"
+               ";; coding : foobar\raa"
+               "\n\n;; coding: foobar\naa"
+               "\r\n\r\n;; coding: foobar\r\naa"
+               "\r\r;; coding: foobar\raa"
+               "\n;;    codincodincoding:\naa")))
+  (test* "to confuse DFA"
+         tdata
+         (map (lambda (i)
+                (with-coding-aware-port
+                 i
+                 (lambda (src wrap)
+                   (let loop ((ch (read-char wrap))
+                              (r  '()))
+                     (if (eof-object? ch)
+                       (list->string (reverse r))
+                       (loop (read-char wrap) (cons ch r)))))))
+              tdata)))
+
 (test-end)
 
