@@ -2,7 +2,7 @@
 ;; testing regexp
 ;;
 
-;; $Id: regexp.scm,v 1.17 2003/10/25 02:42:09 shirok Exp $
+;; $Id: regexp.scm,v 1.19 2004/09/12 21:42:40 shirok Exp $
 
 (use gauche.test)
 (use srfi-1)
@@ -625,6 +625,11 @@
                                      (/ (string-length (rxmatch-substring m 1))
                                         3)))))
 
+(test* "regexp-replace-all" *test-error*
+       (regexp-replace-all #/\d*/ "abcdef" "X"))
+(test* "regexp-replace-all" *test-error*
+       (regexp-replace-all #/\d*/ "123abcdef" "X"))
+
 ;;-------------------------------------------------------------------------
 (test-section "regexp cimatch")
 
@@ -694,4 +699,18 @@
        (let ((str "^(#$%#}{)\\-+?^$"))
          (regmatch? (rxmatch (string->regexp (regexp-quote str)) str))))
 
+;;-------------------------------------------------------------------------
+(test-section "regexp comparison")
+
+(test* "equal #/abc/ #/abc/" #t
+       (equal? #/abc/ #/abc/))
+(test* "not equal #/abc/ #/a(bc)/" #f
+       (equal? #/abc/ #/a(bc)/))
+(test* "not equal #/abc/ #/abc/i" #f
+       (equal? #/abc/ #/abc/i))
+(test* "not equal #/abc/i #/abc/" #f
+       (equal? #/abc/i #/abc/))
+(test* "equal #/abc/i #/abc/i" #t
+       (equal? #/abc/i #/abc/i))
+
 (test-end)
