@@ -2,7 +2,7 @@
 ;; Test for SRFIs
 ;;
 
-;; $Id: srfi.scm,v 1.34 2004/02/26 07:41:41 shirok Exp $
+;; $Id: srfi.scm,v 1.39 2004/11/01 21:51:05 shirok Exp $
 
 (use gauche.test)
 
@@ -17,6 +17,8 @@
        (cond-expand (hogehoge 0) (else 1)))
 (test* "cond-expand" 0
        (cond-expand ((and srfi-0 srfi-1) 0) (else 1)))
+(test* "cond-expand" #t
+       (cond-expand ((and srfi-2 srfi-1) (procedure? xcons)) (else #f)))
 (test* "cond-expand" 0
        (cond-expand ((or hogehoge srfi-1) 0) (else 1)))
 (test* "cond-expand" 0
@@ -521,10 +523,10 @@
        (eval '(program
                (requires no-such-feature))
              (make-module #f)))
-(test* "program (files (empty))" 3
+(test* "program (files (empty))" '(1 . 2)
        (eval '(program
                (files)
-               (code (+ 1 2)))
+               (code (cons 1 2)))
              (make-module #f)))
 (test* "program (files)" 6
        (eval '(program
@@ -781,6 +783,10 @@
        (string-contains-ci "Ma mere l'oye" "Mer"))
 (test* "string-contains-ci" #f
        (string-contains-ci "Ma mere l'oye" "Meer"))
+(test* "string-contains" 15
+       (string-contains "eek -- what a geek." "ee" 12 18))
+(test* "string-contains-ci" 15
+       (string-contains-ci "Eek -- what a geek." "EE" 12 18))
 
 (test* "string-titlecase" "--Capitalize This Sentence."
        (string-titlecase "--capitalize tHIS sentence."))
@@ -1519,19 +1525,33 @@
 
 ;; NB: in Gauche, the round-trip conversion from time -> julian-day -> time
 ;; can't be guaranteed because of the limited precision of julian-day
-;; calcularion.
-'(let1 t0 (make-time time-utc 0 1022191954)
-   (test "julian day number"
-         t0
-         (lambda ()
-           (julian-day->time-utc (time-utc->julian-day t0)))
-         time=?))
-'(let1 t0 (make-time time-utc 0 1022191954)
-   (test "modified julian day number"
-         t0
-         (lambda ()
-           (modified-julian-day->time-utc (time-utc->modified-julian-day t0)))
-         time=?))
+;; calcularion.   We round the nanosecond range.
+(define (round-to-seconds time)
+  (let ((n (time-nanosecond time)))
+    (set! (ref time 'nanosecond) 0)
+    (when (> n 500000000)
+      (add-duration! time (make-time 'time-duration 0 1)))
+    time))
+  
+(let1 t0 (make-time time-utc 0 1022191954)
+  (test "julian day number, via time-utc"
+        t0
+        (lambda ()
+          (round-to-seconds (julian-day->time-utc (time-utc->julian-day t0))))
+        time=?))
+(let1 jd 2453311.0
+  (test "julian day number, via date"
+        jd
+        (lambda ()
+          (date->julian-day (julian-day->date jd)))))
+(let1 t0 (make-time time-utc 0 1022191954)
+  (test "modified julian day number"
+        t0
+        (lambda ()
+          (round-to-seconds
+           (modified-julian-day->time-utc (time-utc->modified-julian-day t0))))
+        time=?))
+
 
 (test* "date->string"
        "2002/05/15 01:23:34.001234567 -1000 3"
